@@ -30,6 +30,10 @@ from threading import Event, Thread, Timer, current_thread
 
 import mozdebug
 import six
+from mozgeckoprofiler import (
+    symbolicate_profile_json,
+    view_gecko_profile,
+)
 from mozserve import Http3Server
 
 try:
@@ -112,13 +116,13 @@ def get_full_group_name(test):
         ancestor_manifest = normsep(test["ancestor_manifest"])
         # Only change the group id if ancestor is not the generated root manifest.
         if "/" in ancestor_manifest:
-            group = "{}:{}".format(ancestor_manifest, group)
+            group = f"{ancestor_manifest}:{group}"
     return group
 
 
 def _cleanup_encoding_repl(m):
     c = m.group(0)
-    return "\\\\" if c == "\\" else "\\x{0:02X}".format(ord(c))
+    return "\\\\" if c == "\\" else f"\\x{ord(c):02X}"
 
 
 def cleanup_encoding(s):
@@ -127,12 +131,12 @@ def cleanup_encoding(s):
     points, etc.  If it is a byte string, it is assumed to be
     UTF-8, but it may not be *correct* UTF-8.  Return a
     sanitized unicode object."""
-    if not isinstance(s, six.string_types):
-        if isinstance(s, six.binary_type):
+    if not isinstance(s, str):
+        if isinstance(s, bytes):
             return six.ensure_str(s)
         else:
-            return six.text_type(s)
-    if isinstance(s, six.binary_type):
+            return str(s)
+    if isinstance(s, bytes):
         s = s.decode("utf-8", "replace")
     # Replace all C0 and C1 control characters with \xNN escapes.
     return _cleanup_encoding_re.sub(_cleanup_encoding_repl, s)
@@ -174,7 +178,7 @@ class XPCShellTestThread(Thread):
         verbose=False,
         usingTSan=False,
         usingCrashReporter=False,
-        **kwargs
+        **kwargs,
     ):
         Thread.__init__(self)
         self.daemon = True
@@ -224,6 +228,7 @@ class XPCShellTestThread(Thread):
         self.timeoutAsPass = kwargs.get("timeoutAsPass")
         self.crashAsPass = kwargs.get("crashAsPass")
         self.conditionedProfileDir = kwargs.get("conditionedProfileDir")
+        self.profiler = kwargs.get("profiler")
         if self.runFailures:
             self.retry = False
 
@@ -371,8 +376,8 @@ class XPCShellTestThread(Thread):
         self.log.info("%s | current directory: %r" % (name, testdir))
         # Show only those environment variables that are changed from
         # the ambient environment.
-        changedEnv = set("%s=%s" % i for i in six.iteritems(self.env)) - set(
-            "%s=%s" % i for i in six.iteritems(os.environ)
+        changedEnv = set("%s=%s" % i for i in self.env.items()) - set(
+            "%s=%s" % i for i in os.environ.items()
         )
         self.log.info("%s | environment: %s" % (name, list(changedEnv)))
         shell_command_tokens = [
@@ -465,7 +470,9 @@ class XPCShellTestThread(Thread):
             # _rootTempDir contains a user.js file, generated by buildPrefsFile
             profile.merge(self._rootTempDir, interpolation=interpolation)
 
-            prefs = self.test_object["prefs"].strip().split()
+            prefs = self.test_object["prefs"].strip()
+            if prefs:
+                prefs = [p.strip() for p in prefs.split("\n")]
             name = self.test_object["id"]
             if self.verbose:
                 self.log.info(
@@ -487,6 +494,24 @@ class XPCShellTestThread(Thread):
         # This is the path set by buildPrefsFile.
         return self.rootPrefsFile
 
+    def updateTestEnvironment(self):
+        # Add additional environment variables from the Manifest file
+        extraEnv = {}
+        if "environment" in self.test_object:
+            extraEnv = self.test_object["environment"].strip().split()
+            self.log.info(
+                "The following extra environment variables will be set:\n  {}".format(
+                    "\n  ".join(extraEnv)
+                )
+            )
+            self.env.update(
+                dict(
+                    parse_key_value(
+                        extraEnv, context="environment variables in manifest"
+                    )
+                )
+            )
+
     @property
     def conditioned_profile_copy(self):
         """Returns a copy of the original conditioned profile that was created."""
@@ -686,7 +711,7 @@ class XPCShellTestThread(Thread):
     def log_line(self, line):
         """Log a line of output (either a parser json object or text output from
         the test process"""
-        if isinstance(line, six.string_types) or isinstance(line, bytes):
+        if isinstance(line, (str, bytes)):
             line = self.fix_text_output(line).rstrip("\r\n")
             self.log.process_output(self.proc_ident, line, command=self.command)
         else:
@@ -808,6 +833,9 @@ class XPCShellTestThread(Thread):
         # Setup per-manifest prefs and write them into the tempdir.
         self.prefsFile = self.updateTestPrefsFile()
 
+        # Setup per-manifest env variables
+        self.updateTestEnvironment()
+
         # The order of the command line is important:
         # 1) Arguments for xpcshell itself
         self.command = self.buildXpcsCmd()
@@ -818,6 +846,9 @@ class XPCShellTestThread(Thread):
         # 3) Arguments for the test file
         self.command.extend(self.buildCmdTestFile(path))
         self.command.extend(["-e", 'const _TEST_NAME = "%s";' % name])
+        self.command.extend(
+            ["-e", 'const _EXPECTED = "%s";' % self.test_object["expected"]]
+        )
 
         # 4) Arguments for code coverage
         if self.jscovdir:
@@ -835,13 +866,21 @@ class XPCShellTestThread(Thread):
             self.env["PYTHON"] = sys.executable
             self.env["BREAKPAD_SYMBOLS_PATH"] = self.symbolsPath
 
-        if self.test_object.get("snap") == "true":
-            self.env["SNAP_NAME"] = "firefox"
-            self.env["SNAP_INSTANCE_NAME"] = "firefox"
-
         if self.test_object.get("subprocess") == "true":
             self.env["PYTHON"] = sys.executable
 
+        if self.profiler:
+            if not self.singleFile:
+                self.log.error(
+                    "The --profiler flag is currently only supported when running a single test"
+                )
+            else:
+                self.env["MOZ_PROFILER_STARTUP"] = "1"
+                profile_path = os.path.join(
+                    self.profileDir, "profile_" + os.path.basename(name) + ".json"
+                )
+                self.env["MOZ_PROFILER_SHUTDOWN"] = profile_path
+
         if (
             self.test_object.get("headless", "true" if self.headless else None)
             == "true"
@@ -858,6 +897,7 @@ class XPCShellTestThread(Thread):
         if not self.interactive and not self.debuggerInfo and not self.jsDebuggerInfo:
             testTimer = Timer(testTimeoutInterval, lambda: self.testTimeout(proc))
             testTimer.start()
+            self.env["MOZ_TEST_TIMEOUT_INTERVAL"] = str(testTimeoutInterval)
 
         proc = None
         process_output = None
@@ -939,6 +979,7 @@ class XPCShellTestThread(Thread):
                 return_code_ok
                 and self.usingCrashReporter
                 and not self.saw_crash_reporter_init
+                and len(process_output) > 0
             )
 
             passed = (
@@ -1024,6 +1065,9 @@ class XPCShellTestThread(Thread):
 
         finally:
             self.postCheck(proc)
+            if self.profiler and self.singleFile:
+                symbolicate_profile_json(profile_path, self.symbolsPath)
+                view_gecko_profile(profile_path)
             self.clean_temp_dirs(path)
 
         if gotSIGINT:
@@ -1037,7 +1081,7 @@ class XPCShellTestThread(Thread):
         self.keep_going = True
 
 
-class XPCShellTests(object):
+class XPCShellTests:
     def __init__(self, log=None):
         """Initializes node status and logger."""
         self.log = log
@@ -1167,7 +1211,7 @@ class XPCShellTests(object):
             else:
                 self.log.error(
                     "no tests to run using specified "
-                    "combination of filters: {}".format(mp.fmt_filters())
+                    f"combination of filters: {mp.fmt_filters()}"
                 )
                 sys.exit(1)
 
@@ -1235,7 +1279,7 @@ class XPCShellTests(object):
             if os.path.isdir(path):
                 profile_data_dir = path
 
-        with open(os.path.join(profile_data_dir, "profiles.json"), "r") as fh:
+        with open(os.path.join(profile_data_dir, "profiles.json")) as fh:
             base_profiles = json.load(fh)["xpcshell"]
 
         # values to use when interpolating preferences
@@ -1337,8 +1381,8 @@ class XPCShellTests(object):
                     self.env["ASAN_SYMBOLIZER_PATH"] = llvmsym
                 else:
                     oldTSanOptions = self.env.get("TSAN_OPTIONS", "")
-                    self.env["TSAN_OPTIONS"] = "external_symbolizer_path={} {}".format(
-                        llvmsym, oldTSanOptions
+                    self.env["TSAN_OPTIONS"] = (
+                        f"external_symbolizer_path={llvmsym} {oldTSanOptions}"
                     )
                 self.log.info("runxpcshelltests.py | using symbolizer at %s" % llvmsym)
             else:
@@ -1404,7 +1448,7 @@ class XPCShellTests(object):
         if not os.path.exists(nodeBin) or not os.path.isfile(nodeBin):
             error = "node not found at MOZ_NODE_PATH %s" % (nodeBin)
             self.log.error(error)
-            raise IOError(error)
+            raise OSError(error)
 
         self.log.info("Found node at %s" % (nodeBin,))
 
@@ -1417,7 +1461,7 @@ class XPCShellTests(object):
             if not os.path.exists(serverJs):
                 error = "%s not found at %s" % (name, serverJs)
                 self.log.error(error)
-                raise IOError(error)
+                raise OSError(error)
 
             # OK, we found our server, let's try to get it running
             self.log.info("Found %s at %s" % (name, serverJs))
@@ -1471,7 +1515,7 @@ class XPCShellTests(object):
         """
         Shut down our node process, if it exists
         """
-        for name, proc in six.iteritems(self.nodeProc):
+        for name, proc in self.nodeProc.items():
             self.log.info("Node %s server shutting down ..." % name)
             if proc.poll() is not None:
                 self.log.info("Node server %s already dead %s" % (name, proc.poll()))
@@ -1635,9 +1679,7 @@ class XPCShellTests(object):
         # create a temp file to help ensure uniqueness
         temp_download_dir = tempfile.mkdtemp()
         self.log.info(
-            "Making temp_download_dir from inside get_conditioned_profile {}".format(
-                temp_download_dir
-            )
+            f"Making temp_download_dir from inside get_conditioned_profile {temp_download_dir}"
         )
         # call condprof's client API to yield our platform-specific
         # conditioned-profile binary
@@ -1684,17 +1726,13 @@ class XPCShellTests(object):
         )
         if not os.path.exists(cond_prof_target_dir):
             self.log.critical(
-                "Can't find target_dir {}, from get_profile()"
-                "temp_download_dir {}, platform {}, scenario {}".format(
-                    cond_prof_target_dir, temp_download_dir, platform, profile_scenario
-                )
+                f"Can't find target_dir {cond_prof_target_dir}, from get_profile()"
+                f"temp_download_dir {temp_download_dir}, platform {platform}, scenario {profile_scenario}"
             )
             raise OSError
 
         self.log.info(
-            "Original self.conditioned_profile_dir is now set: {}".format(
-                self.conditioned_profile_dir
-            )
+            f"Original self.conditioned_profile_dir is now set: {self.conditioned_profile_dir}"
         )
         return self.conditioned_profile_copy
 
@@ -1815,6 +1853,7 @@ class XPCShellTests(object):
         self.conditionedProfile = options.get("conditionedProfile")
         self.repeat = options.get("repeat", 0)
         self.variant = options.get("variant", "")
+        self.profiler = options.get("profiler")
 
         if self.variant == "msix":
             self.appPath = options.get("msixAppPath")
@@ -1857,7 +1896,7 @@ class XPCShellTests(object):
             "can be used to skip tests conditionally:"
         )
         for info in sorted(self.mozInfo.items(), key=lambda item: item[0]):
-            self.log.info("    {key}: {value}".format(key=info[0], value=info[1]))
+            self.log.info(f"    {info[0]}: {info[1]}")
 
         if options.get("self_test"):
             if not self.runSelfTest():
@@ -1927,7 +1966,11 @@ class XPCShellTests(object):
                 break
 
         if installNPM:
-            command = "npm ci"
+            npm = "npm"
+            nodePath = os.environ.get("MOZ_NODE_PATH", "")
+            if nodePath:
+                npm = f"PATH=$PATH:{'/'.join(nodePath.split('/')[:-1])} {'/'.join(nodePath.split('/')[:-1])}/npm"
+            command = f"{npm} ci"
             working_directory = os.path.join(SCRIPT_DIR, "moz-http2")
             result = subprocess.run(
                 command,
@@ -1982,6 +2025,7 @@ class XPCShellTests(object):
             "crashAsPass": self.crashAsPass,
             "conditionedProfileDir": self.conditioned_profile_dir,
             "repeat": self.repeat,
+            "profiler": self.profiler,
         }
 
         if self.sequential:
