@@ -1,32 +1,37 @@
+/* import-globals-from antitracking_head.js */
+
 // This test ensures HasStorageAccess API returns the right value under different
 // scenarios.
 
-/* import-globals-from antitracking_head.js */
-
 var settings = [
   // same-origin no-tracker
   {
     name: "Test whether same-origin non-tracker frame has storage access",
-    topPage: TEST_TOP_PAGE,
-    thirdPartyPage: TEST_DOMAIN + TEST_PATH + "3rdParty.html",
+    topPage: TEST_TOP_PAGE_HTTPS,
+    thirdPartyPage: TEST_DOMAIN_HTTPS + TEST_PATH + "3rdParty.html",
   },
   // 3rd-party no-tracker
   {
     name: "Test whether 3rd-party non-tracker frame has storage access",
-    topPage: TEST_TOP_PAGE,
-    thirdPartyPage: TEST_4TH_PARTY_PAGE,
+    topPage: TEST_TOP_PAGE_HTTPS,
+    thirdPartyPage: TEST_4TH_PARTY_PAGE_HTTPS,
   },
   // 3rd-party no-tracker with permission
   {
-    name:
-      "Test whether 3rd-party non-tracker frame has storage access when storage permission is granted before",
-    topPage: TEST_TOP_PAGE,
-    thirdPartyPage: TEST_4TH_PARTY_PAGE,
+    name: "Test whether 3rd-party non-tracker frame has storage access when storage permission is granted before",
+    topPage: TEST_TOP_PAGE_HTTPS,
+    thirdPartyPage: TEST_4TH_PARTY_PAGE_HTTPS,
     setup: () => {
-      let type = "3rdPartyStorage^http://not-tracking.example.com";
+      let type = "3rdPartyFrameStorage^https://not-tracking.example.com";
       let permission = Services.perms.ALLOW_ACTION;
       let expireType = Services.perms.EXPIRE_SESSION;
-      PermissionTestUtils.add(TEST_DOMAIN, type, permission, expireType, 0);
+      PermissionTestUtils.add(
+        TEST_DOMAIN_HTTPS,
+        type,
+        permission,
+        expireType,
+        0
+      );
 
       registerCleanupFunction(_ => {
         Services.perms.removeAll();
@@ -36,20 +41,25 @@ var settings = [
   // 3rd-party tracker
   {
     name: "Test whether 3rd-party tracker frame has storage access",
-    topPage: TEST_TOP_PAGE,
+    topPage: TEST_TOP_PAGE_HTTPS,
     thirdPartyPage: TEST_3RD_PARTY_PAGE,
   },
   // 3rd-party tracker with permission
   {
-    name:
-      "Test whether 3rd-party tracker frame has storage access when storage access permission is granted before",
-    topPage: TEST_TOP_PAGE,
+    name: "Test whether 3rd-party tracker frame has storage access when storage access permission is granted before",
+    topPage: TEST_TOP_PAGE_HTTPS,
     thirdPartyPage: TEST_3RD_PARTY_PAGE,
     setup: () => {
-      let type = "3rdPartyStorage^https://tracking.example.org";
+      let type = "3rdPartyFrameStorage^https://example.org";
       let permission = Services.perms.ALLOW_ACTION;
       let expireType = Services.perms.EXPIRE_SESSION;
-      PermissionTestUtils.add(TEST_DOMAIN, type, permission, expireType, 0);
+      PermissionTestUtils.add(
+        TEST_DOMAIN_HTTPS,
+        type,
+        permission,
+        expireType,
+        0
+      );
 
       registerCleanupFunction(_ => {
         Services.perms.removeAll();
@@ -59,14 +69,20 @@ var settings = [
   // same-site 3rd-party tracker
   {
     name: "Test whether same-site 3rd-party tracker frame has storage access",
-    topPage: TEST_TOP_PAGE,
-    thirdPartyPage: TEST_ANOTHER_3RD_PARTY_PAGE,
+    topPage: TEST_TOP_PAGE_HTTPS,
+    thirdPartyPage: TEST_ANOTHER_3RD_PARTY_PAGE_HTTPS,
   },
   // same-origin 3rd-party tracker
   {
     name: "Test whether same-origin 3rd-party tracker frame has storage access",
-    topPage: TEST_ANOTHER_3RD_PARTY_DOMAIN + TEST_PATH + "page.html",
-    thirdPartyPage: TEST_ANOTHER_3RD_PARTY_PAGE,
+    topPage: TEST_ANOTHER_3RD_PARTY_DOMAIN_HTTPS + TEST_PATH + "page.html",
+    thirdPartyPage: TEST_ANOTHER_3RD_PARTY_PAGE_HTTPS,
+  },
+  // Insecure 3rd-party tracker
+  {
+    name: "Test whether insecure 3rd-party tracker frame has storage access",
+    topPage: TEST_TOP_PAGE + TEST_PATH + "page.html",
+    thirdPartyPage: TEST_3RD_PARTY_PAGE_HTTP,
   },
 ];
 
@@ -81,6 +97,7 @@ var testCases = [
       true /* 3rd-party tracker with permission */,
       true /* same-site tracker */,
       true /* same-origin tracker */,
+      true /* Insecure context */,
     ],
   },
   {
@@ -88,15 +105,12 @@ var testCases = [
     hasStorageAccess: [
       true /* same-origin non-tracker */,
       false /* 3rd-party non-tracker */,
-      SpecialPowers.Services.prefs.getBoolPref(
-        "network.cookie.rejectForeignWithExceptions.enabled"
-      ) /* 3rd-party tracker with permission */,
+      false /* 3rd-party non-tracker with permission */,
       false /* 3rd-party tracker */,
-      SpecialPowers.Services.prefs.getBoolPref(
-        "network.cookie.rejectForeignWithExceptions.enabled"
-      ) /* 3rd-party non-tracker with permission */,
+      false /* 3rd-party tracker with permission */,
       true /* same-site tracker */,
       true /* same-origin tracker */,
+      false /* Insecure context */,
     ],
   },
   {
@@ -109,6 +123,7 @@ var testCases = [
       false /* 3rd-party tracker with permission */,
       false /* same-site tracker */,
       false /* same-origin tracker */,
+      false /* Insecure context */,
     ],
   },
   {
@@ -121,6 +136,7 @@ var testCases = [
       false /* 3rd-party tracker with permission */,
       true /* same-site tracker */,
       true /* same-origin tracker */,
+      false /* Insecure context */,
     ],
   },
   {
@@ -130,9 +146,10 @@ var testCases = [
       true /* 3rd-party non-tracker */,
       true /* 3rd-party non-tracker with permission */,
       false /* 3rd-party tracker */,
-      true /* 3rd-party tracker with permission */,
+      false /* 3rd-party tracker with permission */,
       true /* same-site tracker */,
       true /* same-origin tracker */,
+      false /* Insecure context */,
     ],
   },
   {
@@ -140,16 +157,17 @@ var testCases = [
     hasStorageAccess: [
       true /* same-origin non-tracker */,
       false /* 3rd-party non-tracker */,
-      true /* 3rd-party non-tracker with permission */,
+      false /* 3rd-party non-tracker with permission */,
       false /* 3rd-party tracker */,
-      true /* 3rd-party tracker with permission */,
+      false /* 3rd-party tracker with permission */,
       true /* same-site tracker */,
       true /* same-origin tracker */,
+      false /* Insecure context */,
     ],
   },
 ];
 
-(function() {
+(function () {
   settings.forEach(setting => {
     if (setting.setup) {
       add_task(async _ => {
@@ -173,7 +191,6 @@ var testCases = [
         cookieBehavior: test.behavior,
         allowList: false,
         callback,
-        extraPrefs: null,
         expectedBlockingNotifications: 0,
         runInPrivateWindow: false,
         iframeSandbox: null,
@@ -186,7 +203,7 @@ var testCases = [
 
     add_task(async _ => {
       await new Promise(resolve => {
-        Services.clearData.deleteData(Ci.nsIClearDataService.CLEAR_ALL, value =>
+        Services.clearData.deleteData(Ci.nsIClearDataService.CLEAR_ALL, () =>
           resolve()
         );
       });
