@@ -25,18 +25,14 @@ ChromeUtils.defineESModuleGetters(this, {
   E10SUtils: "resource://gre/modules/E10SUtils.sys.mjs",
   FileTestUtils: "resource://testing-common/FileTestUtils.sys.mjs",
   FileUtils: "resource://gre/modules/FileUtils.sys.mjs",
+  HttpServer: "resource://testing-common/httpd.sys.mjs",
   MockRegistrar: "resource://testing-common/MockRegistrar.sys.mjs",
+  NetUtil: "resource://gre/modules/NetUtil.sys.mjs",
   PlacesUtils: "resource://gre/modules/PlacesUtils.sys.mjs",
-  PromiseUtils: "resource://gre/modules/PromiseUtils.sys.mjs",
   TelemetryTestUtils: "resource://testing-common/TelemetryTestUtils.sys.mjs",
   TestUtils: "resource://testing-common/TestUtils.sys.mjs",
 });
 
-XPCOMUtils.defineLazyModuleGetters(this, {
-  HttpServer: "resource://testing-common/httpd.js",
-  NetUtil: "resource://gre/modules/NetUtil.jsm",
-});
-
 XPCOMUtils.defineLazyServiceGetter(
   this,
   "gExternalHelperAppService",
@@ -269,7 +265,7 @@ function promiseStartLegacyDownload(aSourceUrl, aOptions) {
   }
 
   if (aOptions && aOptions.launcherPath) {
-    Assert.ok(mimeInfo != null);
+    Assert.notEqual(mimeInfo, null);
 
     let localHandlerApp = Cc[
       "@mozilla.org/uriloader/local-handler-app;1"
@@ -279,9 +275,19 @@ function promiseStartLegacyDownload(aSourceUrl, aOptions) {
     mimeInfo.preferredApplicationHandler = localHandlerApp;
     mimeInfo.preferredAction = Ci.nsIMIMEInfo.useHelperApp;
   }
+  if (aOptions && aOptions.launcherId) {
+    Assert.notEqual(mimeInfo, null);
+
+    let gioHandlerApp = Cc["@mozilla.org/gio-service;1"]
+      .getService(Ci.nsIGIOService)
+      .createHandlerAppFromAppId(aOptions.launcherId);
+
+    mimeInfo.preferredApplicationHandler = gioHandlerApp;
+    mimeInfo.preferredAction = Ci.nsIMIMEInfo.useHelperApp;
+  }
 
   if (aOptions && aOptions.launchWhenSucceeded) {
-    Assert.ok(mimeInfo != null);
+    Assert.notEqual(mimeInfo, null);
 
     mimeInfo.preferredAction = Ci.nsIMIMEInfo.useHelperApp;
   }
@@ -611,7 +617,7 @@ async function promiseVerifyContents(aPath, aExpectedContents) {
         ) {
           // Do not print the entire content string to the test log.
           Assert.equal(contents.length, aExpectedContents.length);
-          Assert.ok(contents == aExpectedContents);
+          Assert.equal(contents, aExpectedContents);
         } else {
           // Print the string if it is short and made of printable characters.
           Assert.equal(contents, aExpectedContents);
@@ -691,12 +697,12 @@ async function promisePartFileReady(aDownload) {
       await promiseTimeout(50);
     } while (!(await IOUtils.exists(aDownload.target.partFilePath)));
   } catch (ex) {
-    if (!(ex instanceof IOUtils.Error)) {
+    if (!DOMException.isInstance(ex)) {
       throw ex;
     }
     // This indicates that the file has been created and cannot be accessed.
     // The specific error might vary with the platform.
-    info("Expected exception while checking existence: " + ex.toString());
+    info("IOUtils exception while checking existence: " + ex.name);
     // Wait some more time to allow the write to complete.
     await promiseTimeout(100);
   }
@@ -710,7 +716,8 @@ async function promisePartFileReady(aDownload) {
  *           keepPartialData: bool,
  *           keepBlockedData: bool,
  *           useLegacySaver: bool,
- *           verdict: string indicating the detailed reason for the block,
+ *           verdict: nsIApplicationReputationService value indicating the reason for the block,
+ *           expectedError: Downloads.Error value indicating the expected error,
  *        }
  * @return {Promise}
  * @resolves The reputation blocked download.
@@ -720,9 +727,10 @@ async function promiseBlockedDownload({
   keepPartialData,
   keepBlockedData,
   useLegacySaver,
-  verdict = Downloads.Error.BLOCK_VERDICT_UNCOMMON,
+  verdict = Ci.nsIApplicationReputationService.VERDICT_UNCOMMON,
+  expectedError = Downloads.Error.BLOCK_VERDICT_UNCOMMON,
 } = {}) {
-  let blockFn = base => ({
+  let blockFn = () => ({
     shouldBlockForReputationCheck: () =>
       Promise.resolve({
         shouldBlock: true,
@@ -760,9 +768,9 @@ async function promiseBlockedDownload({
       throw ex;
     }
     Assert.ok(ex.becauseBlockedByReputationCheck);
-    Assert.equal(ex.reputationCheckVerdict, verdict);
+    Assert.equal(ex.reputationCheckVerdict, expectedError);
     Assert.ok(download.error.becauseBlockedByReputationCheck);
-    Assert.equal(download.error.reputationCheckVerdict, verdict);
+    Assert.equal(download.error.reputationCheckVerdict, expectedError);
   }
 
   Assert.ok(download.stopped);
@@ -792,7 +800,7 @@ function startFakeServer() {
 /**
  * This is an internal reference that should not be used directly by tests.
  */
-var _gDeferResponses = PromiseUtils.defer();
+var _gDeferResponses = Promise.withResolvers();
 
 /**
  * Ensures that all the interruptible requests started after this function is
@@ -820,7 +828,7 @@ function mustInterruptResponses() {
   _gDeferResponses.resolve();
 
   info("Interruptible responses will be blocked midway.");
-  _gDeferResponses = PromiseUtils.defer();
+  _gDeferResponses = Promise.withResolvers();
 }
 
 /**
@@ -1132,7 +1140,7 @@ add_setup(function test_common_initialize() {
       shouldBlockForReputationCheck: () =>
         Promise.resolve({
           shouldBlock: false,
-          verdict: "",
+          verdict: Ci.nsIApplicationReputationService.VERDICT_SAFE,
         }),
       confirmLaunchExecutable: () => Promise.resolve(),
       launchFile: () => Promise.resolve(),
@@ -1163,13 +1171,7 @@ add_setup(function test_common_initialize() {
   // saved to disk without asking for a destination interactively.
   let mock = {
     QueryInterface: ChromeUtils.generateQI(["nsIHelperAppLauncherDialog"]),
-    promptForSaveToFileAsync(
-      aLauncher,
-      aWindowContext,
-      aDefaultFileName,
-      aSuggestedFileExtension,
-      aForcePrompt
-    ) {
+    promptForSaveToFileAsync(aLauncher) {
       // The dialog should create the empty placeholder file.
       let file = getTempFile(TEST_TARGET_FILE_NAME);
       file.create(Ci.nsIFile.NORMAL_FILE_TYPE, FileUtils.PERMS_FILE);
