@@ -2,27 +2,27 @@
  * http://creativecommons.org/publicdomain/zero/1.0/ */
 "use strict";
 
-const { Sampling } = ChromeUtils.import(
-  "resource://gre/modules/components-utils/Sampling.jsm"
+const { Sampling } = ChromeUtils.importESModule(
+  "resource://gre/modules/components-utils/Sampling.sys.mjs"
 );
-const { TelemetryEnvironment } = ChromeUtils.import(
-  "resource://gre/modules/TelemetryEnvironment.jsm"
+
+const { ClientEnvironment } = ChromeUtils.importESModule(
+  "resource://normandy/lib/ClientEnvironment.sys.mjs"
 );
-const { ClientEnvironment } = ChromeUtils.import(
-  "resource://normandy/lib/ClientEnvironment.jsm"
+const { PreferenceExperiments } = ChromeUtils.importESModule(
+  "resource://normandy/lib/PreferenceExperiments.sys.mjs"
 );
-const { PreferenceExperiments } = ChromeUtils.import(
-  "resource://normandy/lib/PreferenceExperiments.jsm"
+const { Uptake } = ChromeUtils.importESModule(
+  "resource://normandy/lib/Uptake.sys.mjs"
 );
-const { Uptake } = ChromeUtils.import("resource://normandy/lib/Uptake.jsm");
-const { BaseAction } = ChromeUtils.import(
-  "resource://normandy/actions/BaseAction.jsm"
+const { BaseAction } = ChromeUtils.importESModule(
+  "resource://normandy/actions/BaseAction.sys.mjs"
 );
-const { PreferenceExperimentAction } = ChromeUtils.import(
-  "resource://normandy/actions/PreferenceExperimentAction.jsm"
+const { PreferenceExperimentAction } = ChromeUtils.importESModule(
+  "resource://normandy/actions/PreferenceExperimentAction.sys.mjs"
 );
-const { NormandyTestUtils } = ChromeUtils.import(
-  "resource://testing-common/NormandyTestUtils.jsm"
+const { NormandyTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/NormandyTestUtils.sys.mjs"
 );
 
 function branchFactory(opts = {}) {
@@ -96,7 +96,7 @@ decorate_task(
     const recipe = prefExperimentRecipeFactory();
     await action.processRecipe(recipe, BaseAction.suitability.FILTER_MATCH);
 
-    Assert.ok(action.log.debug.args.length === 1);
+    Assert.strictEqual(action.log.debug.args.length, 1);
     Assert.deepEqual(action.log.debug.args[0], [
       "User has opted-out of opt-out experiments, disabling action.",
     ]);
@@ -108,7 +108,7 @@ decorate_task(
     ]);
 
     await action.finalize();
-    Assert.ok(action.log.debug.args.length === 2);
+    Assert.strictEqual(action.log.debug.args.length, 2);
     Assert.deepEqual(action.log.debug.args[1], [
       "Skipping post-execution hook for PreferenceExperimentAction because it is disabled.",
     ]);
@@ -154,7 +154,7 @@ decorate_task(
     });
     sinon
       .stub(action, "chooseBranch")
-      .callsFake(async function(slug, branches) {
+      .callsFake(async function (slug, branches) {
         return branches[0];
       });
 
@@ -327,7 +327,7 @@ decorate_task(
     });
     action.chooseBranch = sinon
       .stub()
-      .callsFake(async function(slug, branches) {
+      .callsFake(async function (slug, branches) {
         return branches[0];
       });
 
@@ -454,7 +454,7 @@ decorate_task(
     const action = new PreferenceExperimentAction();
     sinon
       .stub(action, "chooseBranch")
-      .callsFake(async function(slug, branches) {
+      .callsFake(async function (slug, branches) {
         return branches[0];
       });
     await action.processRecipe(recipe, BaseAction.suitability.FILTER_MATCH);
@@ -480,7 +480,6 @@ decorate_task(
         experimentType: "exp",
         userFacingName: "userFacingName",
         userFacingDescription: "userFacingDescription",
-        enrollmentId: activeExperiments[0].enrollmentId,
       },
     ]);
 
@@ -744,8 +743,9 @@ decorate_task(
         let modifiedExperiment = await PreferenceExperiments.get(
           experiment.slug
         );
-        ok(
-          modifiedExperiment.temporaryErrorDeadline != invalidDeadline,
+        Assert.notEqual(
+          modifiedExperiment.temporaryErrorDeadline,
+          invalidDeadline,
           `The temporary failure deadline should be reset for suitabilitiy ${suitability}`
         );
         let deadline = new Date(modifiedExperiment.temporaryErrorDeadline);
@@ -831,7 +831,6 @@ decorate_task(
   withSpy(PreferenceExperiments, "stop"),
   withStub(PreferenceExperimentAction.prototype, "_considerTemporaryError"),
   async function testNoRecipes({
-    stopSpy,
     _considerTemporaryErrorStub,
     prefExperiments: [experiment],
   }) {
