@@ -13,32 +13,31 @@
 #include "nsIAutoCompleteController.h"
 #include "nsIAutoCompletePopup.h"
 #include "nsIDOMEventListener.h"
-#include "nsIFormAutoComplete.h"
 #include "nsCOMPtr.h"
 #include "nsStubMutationObserver.h"
 #include "nsTHashMap.h"
 #include "nsInterfaceHashtable.h"
 #include "nsIDocShell.h"
-#include "nsILoginAutoCompleteSearch.h"
 #include "nsIMutationObserver.h"
 #include "nsIObserver.h"
 #include "nsCycleCollectionParticipant.h"
-#include "nsILoginReputation.h"
 
 class nsFormHistory;
 class nsINode;
 
 namespace mozilla {
+class CancelableRunnable;
+class ErrorResult;
 namespace dom {
 class EventTarget;
-class HTMLInputElement;
+class Element;
 }  // namespace dom
 }  // namespace mozilla
 
 class nsFormFillController final : public nsIFormFillController,
                                    public nsIAutoCompleteInput,
                                    public nsIAutoCompleteSearch,
-                                   public nsIFormAutoCompleteObserver,
+                                   public nsIFormFillCompleteObserver,
                                    public nsIDOMEventListener,
                                    public nsIObserver,
                                    public nsMultiMutationObserver {
@@ -47,7 +46,7 @@ class nsFormFillController final : public nsIFormFillController,
   NS_DECL_NSIFORMFILLCONTROLLER
   NS_DECL_NSIAUTOCOMPLETESEARCH
   NS_DECL_NSIAUTOCOMPLETEINPUT
-  NS_DECL_NSIFORMAUTOCOMPLETEOBSERVER
+  NS_DECL_NSIFORMFILLCOMPLETEOBSERVER
   NS_DECL_NSIDOMEVENTLISTENER
   NS_DECL_NSIOBSERVER
   NS_DECL_NSIMUTATIONOBSERVER
@@ -67,13 +66,13 @@ class nsFormFillController final : public nsIFormFillController,
   MOZ_CAN_RUN_SCRIPT virtual ~nsFormFillController();
 
   MOZ_CAN_RUN_SCRIPT
-  void StartControllingInput(mozilla::dom::HTMLInputElement* aInput);
+  void StartControllingInput(mozilla::dom::Element* aInput);
   MOZ_CAN_RUN_SCRIPT void StopControllingInput();
 
   bool IsFocusedInputControlled() const;
 
   MOZ_CAN_RUN_SCRIPT
-  nsresult HandleFocus(mozilla::dom::HTMLInputElement* aInput);
+  nsresult HandleFocus(mozilla::dom::Element* aInput);
 
   void AttachListeners(mozilla::dom::EventTarget* aEventTarget);
 
@@ -82,17 +81,25 @@ class nsFormFillController final : public nsIFormFillController,
    * StartControllingInput on it.
    */
   MOZ_CAN_RUN_SCRIPT
-  void MaybeStartControllingInput(mozilla::dom::HTMLInputElement* aElement);
+  void MaybeStartControllingInput(mozilla::dom::Element* aElement);
 
-  nsresult PerformInputListAutoComplete(const nsAString& aSearch,
-                                        nsIAutoCompleteResult** aResult);
+  // clears the reference mRestartAfterAttributeChangeTask before running
+  // MaybeStartControllingInput()
+  MOZ_CAN_RUN_SCRIPT
+  void MaybeStartControllingInputScheduled(mozilla::dom::Element* aElement);
+
+  // cancels a scheduled AttributeChangeTask and clears the reference
+  // mRestartAfterAttributeChangeTask
+  MOZ_CAN_RUN_SCRIPT
+  void MaybeCancelAttributeChangeTask();
+
+  void MaybeObserveDataListMutations();
 
   MOZ_CAN_RUN_SCRIPT void RevalidateDataList();
   bool RowMatch(nsFormHistory* aHistory, uint32_t aIndex,
                 const nsAString& aInputName, const nsAString& aInputValue);
 
-  inline nsIDocShell* GetDocShellForInput(
-      mozilla::dom::HTMLInputElement* aInput);
+  inline nsIDocShell* GetDocShellForInput(mozilla::dom::Element* aInput);
 
   void MaybeRemoveMutationObserver(nsINode* aNode);
 
@@ -100,38 +107,25 @@ class nsFormFillController final : public nsIFormFillController,
 
   bool IsTextControl(nsINode* aNode);
 
-  nsresult StartQueryLoginReputation(mozilla::dom::HTMLInputElement* aInput);
-
-  MOZ_CAN_RUN_SCRIPT NS_IMETHODIMP isLoginManagerField(
-      mozilla::dom::HTMLInputElement* aInput, bool* isLoginManagerField);
-
   // members //////////////////////////////////////////
 
   nsCOMPtr<nsIAutoCompleteController> mController;
-  nsCOMPtr<nsILoginAutoCompleteSearch> mLoginManagerAC;
-  nsCOMPtr<nsILoginReputationService> mLoginReputationService;
-  mozilla::dom::HTMLInputElement* mFocusedInput;
+  mozilla::dom::Element* mFocusedElement;
+  RefPtr<mozilla::CancelableRunnable> mRestartAfterAttributeChangeTask;
 
   // mListNode is a <datalist> element which, is set, has the form fill
   // controller as a mutation observer for it.
   nsINode* mListNode;
   nsCOMPtr<nsIAutoCompletePopup> mFocusedPopup;
 
-  // Only used by tests.
-  nsInterfaceHashtable<nsRefPtrHashKey<mozilla::dom::Document>,
-                       nsIAutoCompletePopup>
-      mPopups;
-
   // The observer passed to StartSearch. It will be notified when the search
   // is complete or the data from a datalist changes.
   nsCOMPtr<nsIAutoCompleteObserver> mLastListener;
 
   // This is cleared by StopSearch().
-  nsCOMPtr<nsIFormAutoComplete> mLastFormAutoComplete;
   nsString mLastSearchString;
 
-  nsTHashMap<nsPtrHashKey<const nsINode>, bool> mPwmgrInputs;
-  nsTHashMap<nsPtrHashKey<const nsINode>, bool> mAutofillInputs;
+  nsTHashMap<nsPtrHashKey<const nsINode>, bool> mAutoCompleteInputs;
 
   uint16_t mFocusAfterRightClickThreshold;
   uint32_t mTimeout;
@@ -146,6 +140,25 @@ class nsFormFillController final : public nsIFormFillController,
   bool mPasswordPopupAutomaticallyOpened;
   bool mAutoCompleteActive = false;
   bool mInvalidatePreviousResult = false;
+
+ private:
+  void GetName(mozilla::dom::Element* aInput, nsAString& aValue);
+  void GetValue(mozilla::dom::Element* aInput, nsAString& aValue);
+  mozilla::dom::Element* GetList(mozilla::dom::Element* aInput);
+  bool HasBeenTypePassword(mozilla::dom::Element* aInput);
+  bool ReadOnly(mozilla::dom::Element* aInput) const;
+  uint32_t GetSelectionStartInternal(mozilla::dom::Element* aInput,
+                                     mozilla::ErrorResult& aRv);
+  uint32_t GetSelectionEndInternal(mozilla::dom::Element* aInput,
+                                   mozilla::ErrorResult& aRv);
+
+  MOZ_CAN_RUN_SCRIPT
+  void SetSelectionRange(mozilla::dom::Element* aInput,
+                         uint32_t aSelectionStart, uint32_t aSelectionEnd,
+                         mozilla::ErrorResult& aRv);
+  void SetUserInput(mozilla::dom::Element* aInput, const nsAString& aValue,
+                    nsIPrincipal& aSubjectPrincipal);
+  void EnablePreview(mozilla::dom::Element* aInput);
 };
 
 #endif  // __nsFormFillController__
