@@ -1,7 +1,7 @@
 /* Any copyright is dedicated to the Public Domain.
  * https://creativecommons.org/publicdomain/zero/1.0/ */
 
-add_task(async function() {
+add_task(async function () {
   let destPath = await copyToProfile(
     "formhistory_v3.sqlite",
     "formhistory.sqlite"
@@ -25,5 +25,5 @@ add_task(async function() {
   }
   // check that an entry still exists
   let num = await promiseCountEntries("name-A", "value-A");
-  Assert.ok(num > 0);
+  Assert.greater(num, 0);
 });
