@@ -23,9 +23,22 @@ const CONFIG = [
     base: {
       name: "Application Default Private",
       urls: {
-        search: { base: "https://example.org", searchTermParamName: "q2" },
+        search: {
+          base: "https://example.org",
+          params: [{ name: "pc", value: "{partnerCode}" }],
+          searchTermParamName: "q2",
+        },
       },
     },
+    variants: [
+      {
+        environment: {
+          allLocalesAndRegions: true,
+        },
+        telemetrySuffix: "123",
+        partnerCode: "foo",
+      },
+    ],
   },
   {
     identifier: "otherEngine1",
@@ -105,21 +118,30 @@ add_task(async function test_defaultPrivateEngine() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "appDefault",
+      partnerCode: "",
+      overriddenByThirdParty: false,
       engineId: "appDefault",
       displayName: "Application Default",
       loadPath: "[app]appDefault",
       submissionUrl: "https://example.org/?q1=",
     },
     private: {
-      engineId: "appDefaultPrivate",
+      providerId: "appDefaultPrivate",
+      partnerCode: "foo",
+      overriddenByThirdParty: false,
+      engineId: "appDefaultPrivate-123",
       displayName: "Application Default Private",
       loadPath: "[app]appDefaultPrivate",
-      submissionUrl: "https://example.org/?q2=",
+      submissionUrl: "https://example.org/?pc=foo&q2=",
     },
   });
 
   let promise = promiseDefaultNotification("private");
-  Services.search.defaultPrivateEngine = engine1;
+  await Services.search.setDefaultPrivate(
+    engine1,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
   Assert.equal(
     await promise,
     engine1,
@@ -139,12 +161,18 @@ add_task(async function test_defaultPrivateEngine() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "appDefault",
+      partnerCode: "",
+      overriddenByThirdParty: false,
       engineId: "appDefault",
       displayName: "Application Default",
       loadPath: "[app]appDefault",
       submissionUrl: "https://example.org/?q1=",
     },
     private: {
+      providerId: "otherEngine1",
+      partnerCode: "",
+      overriddenByThirdParty: false,
       engineId: "otherEngine1",
       displayName: "Other Engine 1",
       loadPath: "[app]otherEngine1",
@@ -184,12 +212,18 @@ add_task(async function test_defaultPrivateEngine() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "appDefault",
+      partnerCode: "",
+      overriddenByThirdParty: false,
       engineId: "appDefault",
       displayName: "Application Default",
       loadPath: "[app]appDefault",
       submissionUrl: "https://example.org/?q1=",
     },
     private: {
+      providerId: "otherEngine2",
+      partnerCode: "",
+      overriddenByThirdParty: false,
       engineId: "otherEngine2",
       displayName: "Other Engine 2",
       loadPath: "[app]otherEngine2",
@@ -215,9 +249,11 @@ add_task(async function test_defaultPrivateEngine() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "appDefault",
       engineId: "appDefault",
     },
     private: {
+      providerId: "otherEngine1",
       engineId: "otherEngine1",
     },
   });
@@ -236,15 +272,20 @@ add_task(async function test_defaultPrivateEngine() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "appDefault",
       engineId: "appDefault",
     },
     private: {
-      engineId: "appDefaultPrivate",
+      providerId: "appDefaultPrivate",
+      engineId: "appDefaultPrivate-123",
     },
   });
 
   engine1.hidden = false;
-  Services.search.defaultEngine = engine1;
+  await Services.search.setDefault(
+    engine1,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
   Assert.equal(
     Services.search.defaultPrivateEngine,
     appPrivateDefault,
@@ -253,14 +294,19 @@ add_task(async function test_defaultPrivateEngine() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "otherEngine1",
       engineId: "otherEngine1",
     },
     private: {
-      engineId: "appDefaultPrivate",
+      providerId: "appDefaultPrivate",
+      engineId: "appDefaultPrivate-123",
     },
   });
 
-  Services.search.defaultEngine = appDefault;
+  await Services.search.setDefault(
+    appDefault,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
 });
 
 add_task(async function test_telemetry_private_empty_submission_url() {
@@ -274,9 +320,15 @@ add_task(async function test_telemetry_private_empty_submission_url() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "appDefault",
+      partnerCode: "",
+      overriddenByThirdParty: false,
       engineId: appDefault.telemetryId,
     },
     private: {
+      providerId: "other",
+      partnerCode: "",
+      overriddenByThirdParty: false,
       engineId: "other-simple",
       displayName: "simple",
       loadPath: "[http]localhost/simple.xml",
@@ -284,18 +336,29 @@ add_task(async function test_telemetry_private_empty_submission_url() {
     },
   });
 
-  Services.search.defaultEngine = appDefault;
+  await Services.search.setDefault(
+    appDefault,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
 });
 
 add_task(async function test_defaultPrivateEngine_turned_off() {
-  Services.search.defaultEngine = appDefault;
-  Services.search.defaultPrivateEngine = engine1;
+  await Services.search.setDefault(
+    appDefault,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
+  await Services.search.setDefaultPrivate(
+    engine1,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "appDefault",
       engineId: "appDefault",
     },
     private: {
+      providerId: "otherEngine1",
       engineId: "otherEngine1",
     },
   });
@@ -313,16 +376,21 @@ add_task(async function test_defaultPrivateEngine_turned_off() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "appDefault",
       engineId: "appDefault",
     },
     private: {
+      providerId: "",
       engineId: "",
     },
   });
 
   promise = promiseDefaultNotification("normal");
   let privatePromise = promiseDefaultNotification("private");
-  Services.search.defaultEngine = engine1;
+  await Services.search.setDefault(
+    engine1,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
   Assert.equal(
     await promise,
     engine1,
@@ -346,15 +414,20 @@ add_task(async function test_defaultPrivateEngine_turned_off() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "otherEngine1",
       engineId: "otherEngine1",
     },
     private: {
+      providerId: "",
       engineId: "",
     },
   });
 
   promise = promiseDefaultNotification("private");
-  Services.search.defaultPrivateEngine = engine2;
+  await Services.search.setDefaultPrivate(
+    engine2,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
   Assert.equal(
     await promise,
     engine2,
@@ -381,15 +454,20 @@ add_task(async function test_defaultPrivateEngine_turned_off() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "otherEngine1",
       engineId: "otherEngine1",
     },
     private: {
+      providerId: "otherEngine2",
       engineId: "otherEngine2",
     },
   });
 
   promise = promiseDefaultNotification("private");
-  Services.search.defaultPrivateEngine = engine1;
+  await Services.search.setDefaultPrivate(
+    engine1,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
   Assert.equal(
     await promise,
     engine1,
@@ -408,9 +486,11 @@ add_task(async function test_defaultPrivateEngine_turned_off() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "otherEngine1",
       engineId: "otherEngine1",
     },
     private: {
+      providerId: "otherEngine1",
       engineId: "otherEngine1",
     },
   });
@@ -424,14 +504,22 @@ add_task(async function test_defaultPrivateEngine_ui_turned_off() {
     true
   );
 
-  Services.search.defaultEngine = engine2;
-  Services.search.defaultPrivateEngine = engine1;
+  await Services.search.setDefault(
+    engine2,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
+  await Services.search.setDefaultPrivate(
+    engine1,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "otherEngine2",
       engineId: "otherEngine2",
     },
     private: {
+      providerId: "otherEngine1",
       engineId: "otherEngine1",
     },
   });
@@ -449,15 +537,20 @@ add_task(async function test_defaultPrivateEngine_ui_turned_off() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "otherEngine2",
       engineId: "otherEngine2",
     },
     private: {
+      providerId: "",
       engineId: "",
     },
   });
 
   promise = promiseDefaultNotification("normal");
-  Services.search.defaultPrivateEngine = engine1;
+  await Services.search.setDefaultPrivate(
+    engine1,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
   Assert.equal(
     await promise,
     engine1,
@@ -471,9 +564,11 @@ add_task(async function test_defaultPrivateEngine_ui_turned_off() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "otherEngine1",
       engineId: "otherEngine1",
     },
     private: {
+      providerId: "",
       engineId: "",
     },
   });
@@ -490,14 +585,22 @@ add_task(async function test_defaultPrivateEngine_same_engine_toggle_pref() {
   );
 
   // Set the normal and private engines to be the same
-  Services.search.defaultEngine = engine2;
-  Services.search.defaultPrivateEngine = engine2;
+  await Services.search.setDefault(
+    engine2,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
+  await Services.search.setDefaultPrivate(
+    engine2,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "otherEngine2",
       engineId: "otherEngine2",
     },
     private: {
+      providerId: "otherEngine2",
       engineId: "otherEngine2",
     },
   });
@@ -520,9 +623,11 @@ add_task(async function test_defaultPrivateEngine_same_engine_toggle_pref() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "otherEngine2",
       engineId: "otherEngine2",
     },
     private: {
+      providerId: "",
       engineId: "",
     },
   });
@@ -545,9 +650,11 @@ add_task(async function test_defaultPrivateEngine_same_engine_toggle_pref() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "otherEngine2",
       engineId: "otherEngine2",
     },
     private: {
+      providerId: "otherEngine2",
       engineId: "otherEngine2",
     },
   });
@@ -564,14 +671,22 @@ add_task(async function test_defaultPrivateEngine_same_engine_toggle_ui_pref() {
   );
 
   // Set the normal and private engines to be the same
-  Services.search.defaultEngine = engine2;
-  Services.search.defaultPrivateEngine = engine2;
+  await Services.search.setDefault(
+    engine2,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
+  await Services.search.setDefaultPrivate(
+    engine2,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "otherEngine2",
       engineId: "otherEngine2",
     },
     private: {
+      providerId: "otherEngine2",
       engineId: "otherEngine2",
     },
   });
@@ -594,9 +709,11 @@ add_task(async function test_defaultPrivateEngine_same_engine_toggle_ui_pref() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "otherEngine2",
       engineId: "otherEngine2",
     },
     private: {
+      providerId: "",
       engineId: "",
     },
   });
@@ -619,9 +736,11 @@ add_task(async function test_defaultPrivateEngine_same_engine_toggle_ui_pref() {
 
   await assertGleanDefaultEngine({
     normal: {
+      providerId: "otherEngine2",
       engineId: "otherEngine2",
     },
     private: {
+      providerId: "otherEngine2",
       engineId: "otherEngine2",
     },
   });
