@@ -8,16 +8,19 @@
 #include "gtest/gtest.h"
 #include "mozilla/CycleCollectedJSContext.h"
 #include "mozilla/dom/ScriptSettings.h"
+#include "mozilla/Maybe.h"
 #include "nsITelemetry.h"
 
 class TelemetryTestFixture : public ::testing::Test {
  protected:
-  TelemetryTestFixture() : mCleanGlobal(nullptr) {}
-  virtual void SetUp();
+  TelemetryTestFixture() = default;
+  virtual void SetUp() final;
+  virtual void TestSpecificSetUp() {};
 
-  JSObject* mCleanGlobal;
+  JSObject* mCleanGlobal = nullptr;
 
   nsCOMPtr<nsITelemetry> mTelemetry;
+  bool mSetupCalled = false;
 };
 
 // AutoJSAPI is annotated with MOZ_STACK_CLASS and thus cannot be
@@ -32,8 +35,7 @@ class MOZ_RAII AutoJSContextWithGlobal {
   JSContext* GetJSContext() const;
 
  protected:
-  mozilla::dom::AutoJSAPI mJsAPI;
-  JSContext* mCx;
+  mozilla::Maybe<mozilla::dom::AutoJSAPI> mJsAPI;
 };
 
 #endif  // TelemetryFixture_h_
