@@ -4,36 +4,33 @@
 
 "use strict";
 
-const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-const { TelemetryStorage } = ChromeUtils.import(
-  "resource://gre/modules/TelemetryStorage.jsm"
+const { TelemetryStorage } = ChromeUtils.importESModule(
+  "resource://gre/modules/TelemetryStorage.sys.mjs"
 );
-const { TelemetryUtils } = ChromeUtils.import(
-  "resource://gre/modules/TelemetryUtils.jsm"
+const { TelemetryUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/TelemetryUtils.sys.mjs"
 );
-const { makeFakeAppDir } = ChromeUtils.import(
-  "resource://testing-common/AppData.jsm"
+const { makeFakeAppDir } = ChromeUtils.importESModule(
+  "resource://testing-common/AppData.sys.mjs"
 );
 
 // The name of the pending pings directory outside of the user profile,
 // in the user app data directory.
 const PENDING_PING_DIR_NAME = "Pending Pings";
 
+// Create a directory inside the profile and register it as UAppData, so
+// we can stick fake crash pings inside there. We put it inside the profile
+// just because we know that will get cleaned up after the test runs.
 async function createFakeAppDir() {
-  // Create a directory inside the profile and register it as UAppData, so
-  // we can stick fake crash pings inside there. We put it inside the profile
-  // just because we know that will get cleaned up after the test runs.
-  let profileDir = Services.dirsvc.get("ProfD", Ci.nsIFile);
-
   // Create "<profile>/UAppData/Pending Pings".
-  const pendingPingsPath = OS.Path.join(
-    profileDir.path,
+  const pendingPingsPath = PathUtils.join(
+    PathUtils.profileDir,
     "UAppData",
     PENDING_PING_DIR_NAME
   );
-  await OS.File.makeDir(pendingPingsPath, {
+  await IOUtils.makeDirectory(pendingPingsPath, {
     ignoreExisting: true,
-    from: OS.Constants.Path.profileDir,
+    createAncestors: true,
   });
 
   await makeFakeAppDir();
@@ -63,11 +60,11 @@ add_task(async function test_migrateUnsentPings() {
     },
   ];
   const APP_DATA_DIR = Services.dirsvc.get("UAppData", Ci.nsIFile).path;
-  const APPDATA_PINGS_DIR = OS.Path.join(APP_DATA_DIR, PENDING_PING_DIR_NAME);
+  const APPDATA_PINGS_DIR = PathUtils.join(APP_DATA_DIR, PENDING_PING_DIR_NAME);
 
   // Create some pending pings outside of the user profile.
   for (let ping of PINGS) {
-    const pingPath = OS.Path.join(APPDATA_PINGS_DIR, ping.id + ".json");
+    const pingPath = PathUtils.join(APPDATA_PINGS_DIR, ping.id + ".json");
     await TelemetryStorage.savePingToFile(ping, pingPath, true);
   }
 
@@ -108,9 +105,9 @@ add_task(async function test_migrateUnsentPings() {
     );
 
     // Verify that the pings are no longer outside of the user profile.
-    const pingPath = OS.Path.join(APPDATA_PINGS_DIR, ping.id + ".json");
+    const pingPath = PathUtils.join(APPDATA_PINGS_DIR, ping.id + ".json");
     Assert.ok(
-      !(await OS.File.exists(pingPath)),
+      !(await IOUtils.exists(pingPath)),
       "The ping should not be in the Pending Pings directory anymore."
     );
   }
@@ -118,10 +115,10 @@ add_task(async function test_migrateUnsentPings() {
 
 add_task(async function test_migrateIncompatiblePing() {
   const APP_DATA_DIR = Services.dirsvc.get("UAppData", Ci.nsIFile).path;
-  const APPDATA_PINGS_DIR = OS.Path.join(APP_DATA_DIR, PENDING_PING_DIR_NAME);
+  const APPDATA_PINGS_DIR = PathUtils.join(APP_DATA_DIR, PENDING_PING_DIR_NAME);
 
   // Create a ping incompatible with migration outside of the user profile.
-  const pingPath = OS.Path.join(APPDATA_PINGS_DIR, "incompatible.json");
+  const pingPath = PathUtils.join(APPDATA_PINGS_DIR, "incompatible.json");
   await TelemetryStorage.savePingToFile({ incom: "patible" }, pingPath, true);
 
   // Ensure the pending ping list is empty.
@@ -138,7 +135,7 @@ add_task(async function test_migrateIncompatiblePing() {
   );
 
   Assert.ok(
-    !(await OS.File.exists(pingPath)),
+    !(await IOUtils.exists(pingPath)),
     "The incompatible ping must have been deleted by the migration"
   );
 });
@@ -146,9 +143,9 @@ add_task(async function test_migrateIncompatiblePing() {
 add_task(async function teardown() {
   // Delete the UAppData directory and make sure nothing breaks.
   const APP_DATA_DIR = Services.dirsvc.get("UAppData", Ci.nsIFile).path;
-  await OS.File.removeDir(APP_DATA_DIR, { ignorePermissions: true });
+  await IOUtils.remove(APP_DATA_DIR, { recursive: true });
   Assert.ok(
-    !(await OS.File.exists(APP_DATA_DIR)),
+    !(await IOUtils.exists(APP_DATA_DIR)),
     "The UAppData directory must not exist anymore."
   );
   TelemetryStorage.reset();
