@@ -8,7 +8,12 @@
 
 #include "AutoSQLiteLifetime.h"
 
+#if defined(XP_WIN) && defined(_M_X64) && defined(MOZ_DIAGNOSTIC_ASSERT_ENABLED)
+#  include <windows.h>
+#endif  // XP_WIN && _M_X64 && MOZ_DIAGNOSTIC_ASSERT_ENABLED
+
 #ifdef MOZ_WIDGET_ANDROID
+#  include "mozilla/jni/Utils.h"
 #  ifdef MOZ_PROFILE_GENERATE
 extern "C" int __llvm_profile_dump(void);
 #  endif
@@ -31,10 +36,6 @@ class BootstrapImpl final : public Bootstrap {
 
   virtual void NS_LogTerm() override { ::NS_LogTerm(); }
 
-  virtual void XRE_TelemetryAccumulate(int aID, uint32_t aSample) override {
-    ::XRE_TelemetryAccumulate(aID, aSample);
-  }
-
   virtual void XRE_StartupTimelineRecord(int aEvent,
                                          mozilla::TimeStamp aWhen) override {
     ::XRE_StartupTimelineRecord(aEvent, aWhen);
@@ -54,14 +55,6 @@ class BootstrapImpl final : public Bootstrap {
     return ::XRE_XPCShellMain(argc, argv, envp, aShellData);
   }
 
-  virtual GeckoProcessType XRE_GetProcessType() override {
-    return ::XRE_GetProcessType();
-  }
-
-  virtual void XRE_SetProcessType(const char* aProcessTypeString) override {
-    ::XRE_SetProcessType(aProcessTypeString);
-  }
-
   virtual nsresult XRE_InitChildProcess(
       int argc, char* argv[], const XREChildData* aChildData) override {
     return ::XRE_InitChildProcess(argc, argv, aChildData);
@@ -72,14 +65,11 @@ class BootstrapImpl final : public Bootstrap {
   }
 
 #ifdef MOZ_WIDGET_ANDROID
-  virtual void GeckoStart(JNIEnv* aEnv, char** argv, int argc,
-                          const StaticXREAppData& aAppData, bool xpcshell,
-                          const char* outFilePath) override {
-    ::GeckoStart(aEnv, argv, argc, aAppData, xpcshell, outFilePath);
+  virtual void XRE_SetGeckoThreadEnv(JNIEnv* aEnv) override {
+    mozilla::jni::SetGeckoThreadEnv(aEnv);
   }
 
-  virtual void XRE_SetAndroidChildFds(
-      JNIEnv* aEnv, const XRE_AndroidChildFds& aFds) override {
+  virtual void XRE_SetAndroidChildFds(JNIEnv* aEnv, jintArray aFds) override {
     ::XRE_SetAndroidChildFds(aEnv, aFds);
   }
 
@@ -98,12 +88,6 @@ class BootstrapImpl final : public Bootstrap {
   }
 #endif
 
-#ifdef MOZ_IPDL_TESTS
-  virtual int XRE_RunIPDLTest(int argc, char** argv) override {
-    return ::XRE_RunIPDLTest(argc, argv);
-  }
-#endif
-
 #ifdef MOZ_ENABLE_FORKSERVER
   virtual int XRE_ForkServer(int* argc, char*** argv) override {
     return ::XRE_ForkServer(argc, argv);
@@ -111,6 +95,27 @@ class BootstrapImpl final : public Bootstrap {
 #endif
 };
 
+#if defined(XP_WIN) && defined(_M_X64) && defined(MOZ_DIAGNOSTIC_ASSERT_ENABLED)
+extern "C" uint32_t _tls_index;
+
+extern "C" NS_EXPORT bool XRE_CheckBlockScopeStaticVarInit(
+    uint32_t* aTlsIndex) {
+  // Copy the value of xul's _tls_index for diagnostics.
+  if (aTlsIndex) {
+    *aTlsIndex = _tls_index;
+  }
+
+  // Check that block-scope static variable initialization works. We use
+  // volatile here to keep the compiler honest - we want it to generate the code
+  // that will ensure that only a single thread goes through the lambda.
+  static bool sItWorks = []() -> bool {
+    bool const volatile value = true;
+    return value;
+  }();
+  return sItWorks;
+}
+#endif  // XP_WIN && _M_X64 && MOZ_DIAGNOSTIC_ASSERT_ENABLED
+
 extern "C" NS_EXPORT void NS_FROZENCALL
 XRE_GetBootstrap(Bootstrap::UniquePtr& b) {
   static bool sBootstrapInitialized = false;
