@@ -11,6 +11,7 @@
 
 #include "jsapi.h"
 #include "xpcpublic.h"
+#include "prprf.h"
 
 #include "nsIAppStartup.h"
 #include "nsIFile.h"
@@ -42,18 +43,23 @@
 #ifdef MOZ_BACKGROUNDTASKS
 #  include "mozilla/BackgroundTasks.h"
 #endif
+#include "mozilla/CmdLineAndEnvUtils.h"
 #include "mozilla/Components.h"
 #include "mozilla/Services.h"
 #include "mozilla/Omnijar.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/ProfilerLabels.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/glean/ToolkitXreMetrics.h"
+#include "mozilla/Try.h"
 #include "mozilla/XREAppData.h"
 #include "nsPrintfCString.h"
 
 #ifdef MOZ_THUNDERBIRD
 #  include "nsIPK11TokenDB.h"
 #  include "nsIPK11Token.h"
+#  ifdef XP_MACOSX
+#    include "MacApplicationDelegate.h"
+#  endif
 #endif
 
 #include <stdlib.h>
@@ -76,15 +82,6 @@
 #  include "UIKitDirProvider.h"
 #endif
 
-#if defined(MOZ_SANDBOX)
-#  include "mozilla/SandboxSettings.h"
-#  include "nsID.h"
-#  include "mozilla/Unused.h"
-#  if defined(XP_WIN)
-#    include "sandboxBroker.h"
-#  endif
-#endif
-
 #if defined(XP_MACOSX)
 #  define APP_REGISTRY_NAME "Application Registry"
 #elif defined(XP_WIN)
@@ -95,21 +92,11 @@
 
 #define PREF_OVERRIDE_DIRNAME "preferences"
 
-#if defined(MOZ_SANDBOX)
-static already_AddRefed<nsIFile> GetProcessSandboxTempDir(
-    GeckoProcessType type);
-static nsresult DeleteDirIfExists(nsIFile* dir);
-static bool IsContentSandboxDisabled();
-static const char* GetProcessTempBaseDirKey();
-static already_AddRefed<nsIFile> CreateProcessSandboxTempDir(
-    GeckoProcessType procType);
-#endif
-
 nsXREDirProvider* gDirServiceProvider = nullptr;
 nsIFile* gDataDirHomeLocal = nullptr;
 nsIFile* gDataDirHome = nullptr;
-nsCOMPtr<nsIFile> gDataDirProfileLocal = nullptr;
-nsCOMPtr<nsIFile> gDataDirProfile = nullptr;
+MOZ_RUNINIT nsCOMPtr<nsIFile> gDataDirProfileLocal = nullptr;
+MOZ_RUNINIT nsCOMPtr<nsIFile> gDataDirProfile = nullptr;
 
 // These are required to allow nsXREDirProvider to be usable in xpcshell tests.
 // where gAppData is null.
@@ -131,9 +118,7 @@ static const char* GetAppVendor() {
 }
 #endif
 
-nsXREDirProvider::nsXREDirProvider() : mProfileNotified(false) {
-  gDirServiceProvider = this;
-}
+nsXREDirProvider::nsXREDirProvider() { gDirServiceProvider = this; }
 
 nsXREDirProvider::~nsXREDirProvider() {
   gDirServiceProvider = nullptr;
@@ -148,47 +133,45 @@ already_AddRefed<nsXREDirProvider> nsXREDirProvider::GetSingleton() {
   return do_AddRef(gDirServiceProvider);
 }
 
-nsresult nsXREDirProvider::Initialize(
-    nsIFile* aXULAppDir, nsIFile* aGREDir,
-    nsIDirectoryServiceProvider* aAppProvider) {
+nsresult nsXREDirProvider::Initialize(nsIFile* aXULAppDir, nsIFile* aGREDir) {
   NS_ENSURE_ARG(aXULAppDir);
   NS_ENSURE_ARG(aGREDir);
 
-  mAppProvider = aAppProvider;
   mXULAppDir = aXULAppDir;
   mGREDir = aGREDir;
   nsCOMPtr<nsIFile> binaryPath;
   nsresult rv = XRE_GetBinaryPath(getter_AddRefs(binaryPath));
-  if (NS_FAILED(rv)) return rv;
-  rv = binaryPath->GetParent(getter_AddRefs(mGREBinDir));
-  if (NS_FAILED(rv)) return rv;
-
-  if (!mProfileDir) {
-    nsCOMPtr<nsIDirectoryServiceProvider> app(mAppProvider);
-    if (app) {
-      bool per = false;
-      app->GetFile(NS_APP_USER_PROFILE_50_DIR, &per,
-                   getter_AddRefs(mProfileDir));
-      NS_ASSERTION(per, "NS_APP_USER_PROFILE_50_DIR must be persistent!");
-      NS_ASSERTION(
-          mProfileDir,
-          "NS_APP_USER_PROFILE_50_DIR not defined! This shouldn't happen!");
-    }
-  }
-
-  return NS_OK;
+  NS_ENSURE_SUCCESS(rv, rv);
+  return binaryPath->GetParent(getter_AddRefs(mGREBinDir));
 }
 
 nsresult nsXREDirProvider::SetProfile(nsIFile* aDir, nsIFile* aLocalDir) {
-  NS_ASSERTION(aDir && aLocalDir, "We don't support no-profile apps yet!");
+  MOZ_ASSERT(aDir && aLocalDir, "We don't support no-profile apps!");
+  MOZ_ASSERT(!mProfileDir && !mProfileLocalDir,
+             "You may only set the profile directories once");
 
-  nsresult rv;
-
-  rv = EnsureDirectoryExists(aDir);
-  if (NS_FAILED(rv)) return rv;
+  nsresult rv = EnsureDirectoryExists(aDir);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   rv = EnsureDirectoryExists(aLocalDir);
-  if (NS_FAILED(rv)) return rv;
+  NS_ENSURE_SUCCESS(rv, rv);
+
+#ifndef XP_WIN
+  nsAutoCString profilePath;
+  rv = aDir->GetNativePath(profilePath);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsAutoCString localProfilePath;
+  rv = aLocalDir->GetNativePath(localProfilePath);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (!mozilla::IsUtf8(profilePath) || !mozilla::IsUtf8(localProfilePath)) {
+    PR_fprintf(
+        PR_STDERR,
+        "Error: The profile path is not valid UTF-8. Unable to continue.\n");
+    return NS_ERROR_FAILURE;
+  }
+#endif
 
 #ifdef XP_MACOSX
   bool same;
@@ -265,6 +248,28 @@ nsresult nsXREDirProvider::GetUserProfilesLocalDir(nsIFile** aResult) {
   return NS_OK;
 }
 
+#ifdef MOZ_BACKGROUNDTASKS
+nsresult nsXREDirProvider::GetBackgroundTasksProfilesRootDir(
+    nsIFile** aResult) {
+  nsCOMPtr<nsIFile> file;
+  nsresult rv = GetUserDataDirectory(getter_AddRefs(file), false);
+
+  if (NS_SUCCEEDED(rv)) {
+#  if !defined(XP_UNIX) || defined(XP_MACOSX)
+    // Sibling to regular user "Profiles" directory.
+    rv = file->AppendNative("Background Tasks Profiles"_ns);
+#  endif
+    // We must create the directory here if it does not exist.
+    nsresult tmp = EnsureDirectoryExists(file);
+    if (NS_FAILED(tmp)) {
+      rv = tmp;
+    }
+  }
+  file.swap(*aResult);
+  return rv;
+}
+#endif
+
 #if defined(XP_UNIX) || defined(XP_MACOSX)
 /**
  * Get the directory that is the parent of the system-wide directories
@@ -293,7 +298,7 @@ static nsresult GetSystemParentDirectory(nsIFile** aFile) {
       "/usr/lib/mozilla"_ns
 #    endif
       ;
-  rv = NS_NewNativeLocalFile(dirname, false, getter_AddRefs(localDir));
+  rv = NS_NewNativeLocalFile(dirname, getter_AddRefs(localDir));
 #  endif
 
   if (NS_SUCCEEDED(rv)) {
@@ -306,139 +311,90 @@ static nsresult GetSystemParentDirectory(nsIFile** aFile) {
 NS_IMETHODIMP
 nsXREDirProvider::GetFile(const char* aProperty, bool* aPersistent,
                           nsIFile** aFile) {
-  nsresult rv;
-
-  bool gettingProfile = false;
-
-  if (!strcmp(aProperty, NS_APP_USER_PROFILE_LOCAL_50_DIR)) {
-    // If XRE_NotifyProfile hasn't been called, don't fall through to
-    // mAppProvider on the profile keys.
-    if (!mProfileNotified) return NS_ERROR_FAILURE;
-
-    if (mProfileLocalDir) return mProfileLocalDir->Clone(aFile);
-
-    if (mAppProvider)
-      return mAppProvider->GetFile(aProperty, aPersistent, aFile);
-
-    // This falls through to the case below
-    gettingProfile = true;
-  }
-  if (!strcmp(aProperty, NS_APP_USER_PROFILE_50_DIR) || gettingProfile) {
-    if (!mProfileNotified) return NS_ERROR_FAILURE;
-
-    if (mProfileDir) return mProfileDir->Clone(aFile);
-
-    if (mAppProvider)
-      return mAppProvider->GetFile(aProperty, aPersistent, aFile);
-
-    // If we don't succeed here, bail early so that we aren't reentrant
-    // through the "GetProfileDir" call below.
-    return NS_ERROR_FAILURE;
-  }
-
-  if (mAppProvider) {
-    rv = mAppProvider->GetFile(aProperty, aPersistent, aFile);
-    if (NS_SUCCEEDED(rv) && *aFile) return rv;
-  }
-
   *aPersistent = true;
+  nsresult rv = NS_ERROR_FAILURE;
 
-  if (!strcmp(aProperty, NS_GRE_DIR)) {
-#if defined(MOZ_WIDGET_ANDROID)
-    // On Android, internal files are inside the APK, a zip file, so this
-    // folder doesn't really make sense.
-    return NS_ERROR_FAILURE;
-#else
-    return mGREDir->Clone(aFile);
-#endif
-  } else if (!strcmp(aProperty, NS_GRE_BIN_DIR)) {
-    return mGREBinDir->Clone(aFile);
-  } else if (!strcmp(aProperty, NS_OS_CURRENT_PROCESS_DIR) ||
-             !strcmp(aProperty, NS_APP_INSTALL_CLEANUP_DIR)) {
-    return GetAppDir()->Clone(aFile);
-  }
-
-  rv = NS_ERROR_FAILURE;
   nsCOMPtr<nsIFile> file;
 
-  if (!strcmp(aProperty, NS_APP_PREF_DEFAULTS_50_DIR)) {
-#if defined(MOZ_WIDGET_ANDROID)
+  if (!strcmp(aProperty, NS_APP_USER_PROFILE_LOCAL_50_DIR) ||
+      !strcmp(aProperty, NS_APP_PROFILE_LOCAL_DIR_STARTUP)) {
+    if (mProfileLocalDir) {
+      rv = mProfileLocalDir->Clone(getter_AddRefs(file));
+    } else {
+      // Profile directories are only set up in the parent process.
+      // We don't expect every caller to check if they are in the right process,
+      // so fail immediately to avoid warning spam.
+      NS_WARNING_ASSERTION(!XRE_IsParentProcess(),
+                           "tried to get profile in parent too early");
+      return NS_ERROR_FAILURE;
+    }
+  } else if (!strcmp(aProperty, NS_APP_USER_PROFILE_50_DIR) ||
+             !strcmp(aProperty, NS_APP_PROFILE_DIR_STARTUP)) {
+    rv = GetProfileStartupDir(getter_AddRefs(file));
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
+  } else if (!strcmp(aProperty, NS_GRE_DIR)) {
+    // On Android, internal files are inside the APK, a zip file, so this
+    // folder doesn't really make sense.
+#if !defined(MOZ_WIDGET_ANDROID)
+    rv = mGREDir->Clone(getter_AddRefs(file));
+#endif  // !defined(MOZ_WIDGET_ANDROID)
+  } else if (!strcmp(aProperty, NS_GRE_BIN_DIR)) {
+    rv = mGREBinDir->Clone(getter_AddRefs(file));
+  } else if (!strcmp(aProperty, NS_OS_CURRENT_PROCESS_DIR) ||
+             !strcmp(aProperty, NS_APP_INSTALL_CLEANUP_DIR)) {
+    rv = GetAppDir()->Clone(getter_AddRefs(file));
+  } else if (!strcmp(aProperty, NS_APP_PREF_DEFAULTS_50_DIR)) {
     // Same as NS_GRE_DIR
-    return NS_ERROR_FAILURE;
-#else
+#if !defined(MOZ_WIDGET_ANDROID)
     // return the GRE default prefs directory here, and the app default prefs
     // directory (if applicable) in NS_APP_PREFS_DEFAULTS_DIR_LIST.
     rv = mGREDir->Clone(getter_AddRefs(file));
-    if (NS_SUCCEEDED(rv)) {
-      rv = file->AppendNative("defaults"_ns);
-      if (NS_SUCCEEDED(rv)) rv = file->AppendNative("pref"_ns);
-    }
-#endif
+    NS_ENSURE_SUCCESS(rv, rv);
+    rv = file->AppendNative("defaults"_ns);
+    NS_ENSURE_SUCCESS(rv, rv);
+    rv = file->AppendNative("pref"_ns);
+#endif  // !defined(MOZ_WIDGET_ANDROID)
   } else if (!strcmp(aProperty, NS_APP_APPLICATION_REGISTRY_DIR) ||
              !strcmp(aProperty, XRE_USER_APP_DATA_DIR)) {
     rv = GetUserAppDataDirectory(getter_AddRefs(file));
   }
 #if defined(XP_UNIX) || defined(XP_MACOSX)
   else if (!strcmp(aProperty, XRE_SYS_NATIVE_MANIFESTS)) {
-    nsCOMPtr<nsIFile> localDir;
-
-    rv = ::GetSystemParentDirectory(getter_AddRefs(localDir));
-    if (NS_SUCCEEDED(rv)) {
-      localDir.swap(file);
-    }
+    rv = ::GetSystemParentDirectory(getter_AddRefs(file));
   } else if (!strcmp(aProperty, XRE_USER_NATIVE_MANIFESTS)) {
-    nsCOMPtr<nsIFile> localDir;
-    rv = GetUserDataDirectoryHome(getter_AddRefs(localDir), false);
-    if (NS_SUCCEEDED(rv)) {
+    rv = GetUserDataDirectoryHome(getter_AddRefs(file), false);
+    NS_ENSURE_SUCCESS(rv, rv);
 #  if defined(XP_MACOSX)
-      rv = localDir->AppendNative("Mozilla"_ns);
-#  else
-      rv = localDir->AppendNative(".mozilla"_ns);
-#  endif
-    }
-    if (NS_SUCCEEDED(rv)) {
-      localDir.swap(file);
-    }
+    rv = file->AppendNative("Mozilla"_ns);
+#  else   // defined(XP_MACOSX)
+    rv = file->AppendNative(".mozilla"_ns);
+#  endif  // defined(XP_MACOSX)
   }
-#endif
+#endif  // defined(XP_UNIX) || defined(XP_MACOSX)
   else if (!strcmp(aProperty, XRE_UPDATE_ROOT_DIR)) {
     rv = GetUpdateRootDir(getter_AddRefs(file));
   } else if (!strcmp(aProperty, XRE_OLD_UPDATE_ROOT_DIR)) {
     rv = GetUpdateRootDir(getter_AddRefs(file), true);
   } else if (!strcmp(aProperty, NS_APP_APPLICATION_REGISTRY_FILE)) {
     rv = GetUserAppDataDirectory(getter_AddRefs(file));
-    if (NS_SUCCEEDED(rv))
-      rv = file->AppendNative(nsLiteralCString(APP_REGISTRY_NAME));
+    NS_ENSURE_SUCCESS(rv, rv);
+    rv = file->AppendNative(nsLiteralCString(APP_REGISTRY_NAME));
   } else if (!strcmp(aProperty, NS_APP_USER_PROFILES_ROOT_DIR)) {
     rv = GetUserProfilesRootDir(getter_AddRefs(file));
   } else if (!strcmp(aProperty, NS_APP_USER_PROFILES_LOCAL_ROOT_DIR)) {
     rv = GetUserProfilesLocalDir(getter_AddRefs(file));
   } else if (!strcmp(aProperty, XRE_EXECUTABLE_FILE)) {
-    nsCOMPtr<nsIFile> lf;
-    rv = XRE_GetBinaryPath(getter_AddRefs(lf));
-    if (NS_SUCCEEDED(rv)) file = lf;
-  }
-
-  else if (!strcmp(aProperty, NS_APP_PROFILE_DIR_STARTUP) && mProfileDir) {
-    return mProfileDir->Clone(aFile);
-  } else if (!strcmp(aProperty, NS_APP_PROFILE_LOCAL_DIR_STARTUP)) {
-    if (mProfileLocalDir) return mProfileLocalDir->Clone(aFile);
-
-    if (mProfileDir) return mProfileDir->Clone(aFile);
-
-    if (mAppProvider)
-      return mAppProvider->GetFile(NS_APP_PROFILE_DIR_STARTUP, aPersistent,
-                                   aFile);
+    rv = XRE_GetBinaryPath(getter_AddRefs(file));
   }
 #if defined(XP_UNIX) || defined(XP_MACOSX)
   else if (!strcmp(aProperty, XRE_SYS_LOCAL_EXTENSION_PARENT_DIR)) {
 #  ifdef ENABLE_SYSTEM_EXTENSION_DIRS
-    return GetSystemExtensionsDirectory(aFile);
-#  else
-    return NS_ERROR_FAILURE;
+    rv = GetSystemExtensionsDirectory(getter_AddRefs(file));
 #  endif
   }
-#endif
+#endif  // defined(XP_UNIX) || defined(XP_MACOSX)
 #if defined(XP_UNIX) && !defined(XP_MACOSX)
   else if (!strcmp(aProperty, XRE_SYS_SHARE_EXTENSION_PARENT_DIR)) {
 #  ifdef ENABLE_SYSTEM_EXTENSION_DIRS
@@ -447,96 +403,76 @@ nsXREDirProvider::GetFile(const char* aProperty, bool* aPersistent,
 #    else
     static const char* const sysLExtDir = "/usr/share/mozilla/extensions";
 #    endif
-    return NS_NewNativeLocalFile(nsDependentCString(sysLExtDir), false, aFile);
-#  else
-    return NS_ERROR_FAILURE;
+    rv = NS_NewNativeLocalFile(nsDependentCString(sysLExtDir),
+                               getter_AddRefs(file));
 #  endif
   }
-#endif
+#endif  // defined(XP_UNIX) && !defined(XP_MACOSX)
   else if (!strcmp(aProperty, XRE_USER_SYS_EXTENSION_DIR)) {
 #ifdef ENABLE_SYSTEM_EXTENSION_DIRS
-    return GetSysUserExtensionsDirectory(aFile);
-#else
-    return NS_ERROR_FAILURE;
+    rv = GetSysUserExtensionsDirectory(getter_AddRefs(file));
 #endif
   } else if (!strcmp(aProperty, XRE_USER_RUNTIME_DIR)) {
 #if defined(XP_UNIX)
     nsPrintfCString path("/run/user/%d/%s/", getuid(), GetAppName());
     ToLowerCase(path);
-    return NS_NewNativeLocalFile(path, false, aFile);
-#else
-    return NS_ERROR_FAILURE;
+    rv = NS_NewNativeLocalFile(path, getter_AddRefs(file));
 #endif
   } else if (!strcmp(aProperty, XRE_APP_DISTRIBUTION_DIR)) {
     bool persistent = false;
     rv = GetFile(NS_GRE_DIR, &persistent, getter_AddRefs(file));
-    if (NS_SUCCEEDED(rv)) rv = file->AppendNative("distribution"_ns);
+    NS_ENSURE_SUCCESS(rv, rv);
+    rv = file->AppendNative("distribution"_ns);
   } else if (!strcmp(aProperty, XRE_APP_FEATURES_DIR)) {
     rv = GetAppDir()->Clone(getter_AddRefs(file));
-    if (NS_SUCCEEDED(rv)) rv = file->AppendNative("features"_ns);
+    NS_ENSURE_SUCCESS(rv, rv);
+    rv = file->AppendNative("features"_ns);
   } else if (!strcmp(aProperty, XRE_ADDON_APP_DIR)) {
     nsCOMPtr<nsIDirectoryServiceProvider> dirsvc(
         do_GetService("@mozilla.org/file/directory_service;1", &rv));
-    if (NS_FAILED(rv)) return rv;
+    NS_ENSURE_SUCCESS(rv, rv);
     bool unused;
     rv = dirsvc->GetFile("XCurProcD", &unused, getter_AddRefs(file));
-  }
-#if defined(MOZ_SANDBOX)
-  else if (!strcmp(aProperty, NS_APP_CONTENT_PROCESS_TEMP_DIR)) {
-    if (!mContentTempDir && NS_FAILED((rv = LoadContentProcessTempDir()))) {
+  } else if (!strcmp(aProperty, NS_APP_USER_CHROME_DIR)) {
+    // It isn't clear why this uses GetProfileStartupDir instead of
+    // GetProfileDir. It could theoretically matter in a non-main
+    // process where some other directory provider has defined
+    // NS_APP_USER_PROFILE_50_DIR. In that scenario, using
+    // GetProfileStartupDir means this will fail instead of succeed.
+    rv = GetProfileStartupDir(getter_AddRefs(file));
+    if (NS_FAILED(rv)) {
       return rv;
     }
-    rv = mContentTempDir->Clone(getter_AddRefs(file));
-  }
-#endif  // defined(MOZ_SANDBOX)
-  else if (NS_SUCCEEDED(GetProfileStartupDir(getter_AddRefs(file)))) {
-    // We need to allow component, xpt, and chrome registration to
-    // occur prior to the profile-after-change notification.
-    if (!strcmp(aProperty, NS_APP_USER_CHROME_DIR)) {
-      rv = file->AppendNative("chrome"_ns);
+    rv = file->AppendNative("chrome"_ns);
+  } else if (!strcmp(aProperty, NS_APP_PREFS_50_DIR)) {
+    rv = GetProfileDir(getter_AddRefs(file));
+    if (NS_FAILED(rv)) {
+      return rv;
     }
-  }
-
-  if (NS_SUCCEEDED(rv) && file) {
-    file.forget(aFile);
-    return NS_OK;
-  }
-
-  bool ensureFilePermissions = false;
-
-  if (NS_SUCCEEDED(GetProfileDir(getter_AddRefs(file)))) {
-    if (!strcmp(aProperty, NS_APP_PREFS_50_DIR)) {
-      rv = NS_OK;
-    } else if (!strcmp(aProperty, NS_APP_PREFS_50_FILE)) {
-      rv = file->AppendNative("prefs.js"_ns);
-    } else if (!strcmp(aProperty, NS_APP_PREFS_OVERRIDE_DIR)) {
-      rv = mProfileDir->Clone(getter_AddRefs(file));
-      nsresult tmp =
-          file->AppendNative(nsLiteralCString(PREF_OVERRIDE_DIRNAME));
-      if (NS_FAILED(tmp)) {
-        rv = tmp;
-      }
-      tmp = EnsureDirectoryExists(file);
-      if (NS_FAILED(tmp)) {
-        rv = tmp;
-      }
+  } else if (!strcmp(aProperty, NS_APP_PREFS_50_FILE)) {
+    rv = GetProfileDir(getter_AddRefs(file));
+    if (NS_FAILED(rv)) {
+      return rv;
     }
-  }
-  if (NS_FAILED(rv) || !file) return NS_ERROR_FAILURE;
-
-  if (ensureFilePermissions) {
-    bool fileToEnsureExists;
-    bool isWritable;
-    if (NS_SUCCEEDED(file->Exists(&fileToEnsureExists)) && fileToEnsureExists &&
-        NS_SUCCEEDED(file->IsWritable(&isWritable)) && !isWritable) {
-      uint32_t permissions;
-      if (NS_SUCCEEDED(file->GetPermissions(&permissions))) {
-        rv = file->SetPermissions(permissions | 0600);
-        NS_ASSERTION(NS_SUCCEEDED(rv), "failed to ensure file permissions");
-      }
+    rv = file->AppendNative("prefs.js"_ns);
+  } else if (!strcmp(aProperty, NS_APP_PREFS_OVERRIDE_DIR)) {
+    rv = GetProfileDir(getter_AddRefs(file));
+    if (NS_FAILED(rv)) {
+      return rv;
     }
+    rv = file->AppendNative(nsLiteralCString(PREF_OVERRIDE_DIRNAME));
+    NS_ENSURE_SUCCESS(rv, rv);
+    rv = EnsureDirectoryExists(file);
+  } else {
+    // We don't know anything about this property. Fail without warning, because
+    // otherwise we'll get too much warning spam due to
+    // nsDirectoryService::Get() trying everything it gets with every provider.
+    return NS_ERROR_FAILURE;
   }
 
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_TRUE(file, NS_ERROR_FAILURE);
+
   file.forget(aFile);
   return NS_OK;
 }
@@ -559,206 +495,6 @@ static void LoadDirIntoArray(nsIFile* dir, const char* const* aAppendList,
   }
 }
 
-NS_IMETHODIMP
-nsXREDirProvider::GetFiles(const char* aProperty,
-                           nsISimpleEnumerator** aResult) {
-  nsresult rv;
-
-  nsCOMPtr<nsISimpleEnumerator> appEnum;
-  nsCOMPtr<nsIDirectoryServiceProvider2> appP2(do_QueryInterface(mAppProvider));
-  if (appP2) {
-    rv = appP2->GetFiles(aProperty, getter_AddRefs(appEnum));
-    if (NS_FAILED(rv)) {
-      appEnum = nullptr;
-    } else if (rv != NS_SUCCESS_AGGREGATE_RESULT) {
-      appEnum.forget(aResult);
-      return NS_OK;
-    }
-  }
-
-  nsCOMPtr<nsISimpleEnumerator> xreEnum;
-  rv = GetFilesInternal(aProperty, getter_AddRefs(xreEnum));
-  if (NS_FAILED(rv)) {
-    if (appEnum) {
-      appEnum.forget(aResult);
-      return NS_SUCCESS_AGGREGATE_RESULT;
-    }
-
-    return rv;
-  }
-
-  rv = NS_NewUnionEnumerator(aResult, appEnum, xreEnum);
-  if (NS_FAILED(rv)) return rv;
-
-  return NS_SUCCESS_AGGREGATE_RESULT;
-}
-
-#if defined(MOZ_SANDBOX)
-
-static const char* GetProcessTempBaseDirKey() {
-#  if defined(XP_WIN)
-  return NS_WIN_LOW_INTEGRITY_TEMP_BASE;
-#  else
-  return NS_OS_TEMP_DIR;
-#  endif
-}
-
-//
-// Sets mContentTempDir so that it refers to the appropriate temp dir.
-// If the sandbox is enabled, NS_APP_CONTENT_PROCESS_TEMP_DIR, otherwise
-// NS_OS_TEMP_DIR is used.
-//
-nsresult nsXREDirProvider::LoadContentProcessTempDir() {
-  // The parent is responsible for creating the sandbox temp dir.
-  if (XRE_IsParentProcess()) {
-    mContentProcessSandboxTempDir =
-        CreateProcessSandboxTempDir(GeckoProcessType_Content);
-    mContentTempDir = mContentProcessSandboxTempDir;
-  } else {
-    mContentTempDir = !IsContentSandboxDisabled()
-                          ? GetProcessSandboxTempDir(GeckoProcessType_Content)
-                          : nullptr;
-  }
-
-  if (!mContentTempDir) {
-    nsresult rv =
-        NS_GetSpecialDirectory(NS_OS_TEMP_DIR, getter_AddRefs(mContentTempDir));
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      return rv;
-    }
-  }
-
-  return NS_OK;
-}
-
-static bool IsContentSandboxDisabled() {
-  return !mozilla::BrowserTabsRemoteAutostart() ||
-         (!mozilla::IsContentSandboxEnabled());
-}
-
-//
-// If a process sandbox temp dir is to be used, returns an nsIFile
-// for the directory. Returns null if an error occurs.
-//
-static already_AddRefed<nsIFile> GetProcessSandboxTempDir(
-    GeckoProcessType type) {
-  nsCOMPtr<nsIFile> localFile;
-
-  nsresult rv = NS_GetSpecialDirectory(GetProcessTempBaseDirKey(),
-                                       getter_AddRefs(localFile));
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return nullptr;
-  }
-
-  MOZ_ASSERT(type == GeckoProcessType_Content);
-
-  const char* prefKey = "security.sandbox.content.tempDirSuffix";
-  nsAutoString tempDirSuffix;
-  rv = mozilla::Preferences::GetString(prefKey, tempDirSuffix);
-  if (NS_WARN_IF(NS_FAILED(rv)) || tempDirSuffix.IsEmpty()) {
-    return nullptr;
-  }
-
-  rv = localFile->Append(u"Temp-"_ns + tempDirSuffix);
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return nullptr;
-  }
-
-  return localFile.forget();
-}
-
-//
-// Create a temporary directory for use from sandboxed processes.
-// Only called in the parent. The path is derived from a UUID stored in a
-// pref which is available to content processes. Returns null
-// if the content sandbox is disabled or if an error occurs.
-//
-static already_AddRefed<nsIFile> CreateProcessSandboxTempDir(
-    GeckoProcessType procType) {
-  if ((procType == GeckoProcessType_Content) && IsContentSandboxDisabled()) {
-    return nullptr;
-  }
-
-  MOZ_ASSERT(procType == GeckoProcessType_Content);
-
-  // Get (and create if blank) temp directory suffix pref.
-  const char* pref = "security.sandbox.content.tempDirSuffix";
-
-  nsresult rv;
-  nsAutoString tempDirSuffix;
-  mozilla::Preferences::GetString(pref, tempDirSuffix);
-
-  if (tempDirSuffix.IsEmpty()) {
-    nsID uuid;
-    rv = nsID::GenerateUUIDInPlace(uuid);
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      return nullptr;
-    }
-
-    char uuidChars[NSID_LENGTH];
-    uuid.ToProvidedString(uuidChars);
-    tempDirSuffix.AssignASCII(uuidChars, NSID_LENGTH);
-#  ifdef XP_UNIX
-    // Braces in a path are somewhat annoying to deal with
-    // and pretty alien on Unix
-    tempDirSuffix.StripChars(u"{}");
-#  endif
-
-    // Save the pref
-    rv = mozilla::Preferences::SetString(pref, tempDirSuffix);
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      // If we fail to save the pref we don't want to create the temp dir,
-      // because we won't be able to clean it up later.
-      return nullptr;
-    }
-
-    nsCOMPtr<nsIPrefService> prefsvc = mozilla::Preferences::GetService();
-    if (!prefsvc || NS_FAILED((rv = prefsvc->SavePrefFile(nullptr)))) {
-      // Again, if we fail to save the pref file we might not be able to clean
-      // up the temp directory, so don't create one.  Note that in the case
-      // the preference values allows an off main thread save, the successful
-      // return from the call doesn't mean we actually saved the file.  See
-      // bug 1364496 for details.
-      NS_WARNING("Failed to save pref file, cannot create temp dir.");
-      return nullptr;
-    }
-  }
-
-  nsCOMPtr<nsIFile> sandboxTempDir = GetProcessSandboxTempDir(procType);
-  if (!sandboxTempDir) {
-    NS_WARNING("Failed to determine sandbox temp dir path.");
-    return nullptr;
-  }
-
-  // Remove the directory. It may exist due to a previous crash.
-  if (NS_FAILED(DeleteDirIfExists(sandboxTempDir))) {
-    NS_WARNING("Failed to reset sandbox temp dir.");
-    return nullptr;
-  }
-
-  // Create the directory
-  rv = sandboxTempDir->Create(nsIFile::DIRECTORY_TYPE, 0700);
-  if (NS_FAILED(rv)) {
-    NS_WARNING("Failed to create sandbox temp dir.");
-    return nullptr;
-  }
-
-  return sandboxTempDir.forget();
-}
-
-static nsresult DeleteDirIfExists(nsIFile* dir) {
-  if (dir) {
-    // Don't return an error if the directory doesn't exist.
-    nsresult rv = dir->Remove(/* aRecursive */ true);
-    if (NS_FAILED(rv) && rv != NS_ERROR_FILE_NOT_FOUND) {
-      return rv;
-    }
-  }
-  return NS_OK;
-}
-
-#endif  // defined(MOZ_SANDBOX)
-
 static const char* const kAppendPrefDir[] = {"defaults", "preferences",
                                              nullptr};
 #ifdef MOZ_BACKGROUNDTASKS
@@ -766,9 +502,10 @@ static const char* const kAppendBackgroundTasksPrefDir[] = {
     "defaults", "backgroundtasks", nullptr};
 #endif
 
-nsresult nsXREDirProvider::GetFilesInternal(const char* aProperty,
-                                            nsISimpleEnumerator** aResult) {
-  nsresult rv = NS_OK;
+NS_IMETHODIMP
+nsXREDirProvider::GetFiles(const char* aProperty,
+                           nsISimpleEnumerator** aResult) {
+  nsresult rv = NS_ERROR_FAILURE;
   *aResult = nullptr;
 
   if (!strcmp(aProperty, NS_APP_PREFS_DEFAULTS_DIR_LIST)) {
@@ -792,40 +529,27 @@ nsresult nsXREDirProvider::GetFilesInternal(const char* aProperty,
     LoadDirIntoArray(mXULAppDir, kAppendChromeDir, directories);
 
     rv = NS_NewArrayEnumerator(aResult, directories, NS_GET_IID(nsIFile));
-  } else
-    rv = NS_ERROR_FAILURE;
+  }
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  return rv;
+  return NS_SUCCESS_AGGREGATE_RESULT;
 }
 
 NS_IMETHODIMP
 nsXREDirProvider::GetDirectory(nsIFile** aResult) {
   NS_ENSURE_TRUE(mProfileDir, NS_ERROR_NOT_INITIALIZED);
-
   return mProfileDir->Clone(aResult);
 }
 
 void nsXREDirProvider::InitializeUserPrefs() {
   if (!mPrefsInitialized) {
-    // Temporarily set mProfileNotified to true so that the preference service
-    // can access the profile directory during initialization. Afterwards, clear
-    // it so that no other code can inadvertently access it until we get to
-    // profile-do-change.
-    mozilla::AutoRestore<bool> ar(mProfileNotified);
-    mProfileNotified = true;
-
     mozilla::Preferences::InitializeUserPrefs();
   }
 }
 
 void nsXREDirProvider::FinishInitializingUserPrefs() {
   if (!mPrefsInitialized) {
-    // See InitializeUserPrefs above.
-    mozilla::AutoRestore<bool> ar(mProfileNotified);
-    mProfileNotified = true;
-
     mozilla::Preferences::FinishInitializingUserPrefs();
-
     mPrefsInitialized = true;
   }
 }
@@ -834,12 +558,12 @@ NS_IMETHODIMP
 nsXREDirProvider::DoStartup() {
   nsresult rv;
 
-  if (!mProfileNotified) {
+  if (!mAppStarted) {
     nsCOMPtr<nsIObserverService> obsSvc =
         mozilla::services::GetObserverService();
     if (!obsSvc) return NS_ERROR_FAILURE;
 
-    mProfileNotified = true;
+    mAppStarted = true;
 
     /*
        Make sure we've setup prefs before profile-do-change to be able to use
@@ -876,17 +600,6 @@ nsXREDirProvider::DoStartup() {
       }
     }
 
-#if defined(MOZ_SANDBOX) && defined(XP_WIN)
-    // Call SandboxBroker to initialize things that depend on Gecko machinery
-    // like the directory provider. We insert this initialization code here
-    // (rather than in XRE_mainRun) because we need NS_APP_USER_PROFILE_50_DIR
-    // to be known and so that any child content processes spawned by extensions
-    // from the notifications below will have all the requisite directories
-    // white-listed for read/write access. An example of this is the
-    // tor-launcher launching the network configuration window. See bug 1485836.
-    mozilla::SandboxBroker::GeckoDependentInitialize();
-#endif
-
 #ifdef MOZ_THUNDERBIRD
     bool bgtaskMode = false;
 #  ifdef MOZ_BACKGROUNDTASKS
@@ -895,6 +608,11 @@ nsXREDirProvider::DoStartup() {
     if (!bgtaskMode &&
         mozilla::Preferences::GetBool(
             "security.prompt_for_master_password_on_startup", false)) {
+#  ifdef XP_MACOSX
+      // Ensure the application is initialized before the prompt is triggered.
+      // Note: calling InitializeMacApp more than once does nothing.
+      InitializeMacApp();
+#  endif
       // Prompt for the master password prior to opening application windows,
       // to avoid the race that triggers multiple prompts (see bug 177175).
       // We use this code until we have a better solution, possibly as
@@ -950,18 +668,9 @@ nsXREDirProvider::DoStartup() {
       else
         mode = 2;
     }
-    mozilla::Telemetry::Accumulate(mozilla::Telemetry::SAFE_MODE_USAGE, mode);
+    mozilla::glean::gecko::safe_mode_usage.AccumulateSingleSample(mode);
 
     obsSvc->NotifyObservers(nullptr, "profile-initial-state", nullptr);
-
-#if defined(MOZ_SANDBOX)
-    // Makes sure the content temp dir has been loaded if it hasn't been
-    // already. In the parent this ensures it has been created before we attempt
-    // to start any content processes.
-    if (!mContentTempDir) {
-      mozilla::Unused << NS_WARN_IF(NS_FAILED(LoadContentProcessTempDir()));
-    }
-#endif
   }
   return NS_OK;
 }
@@ -969,7 +678,7 @@ nsXREDirProvider::DoStartup() {
 void nsXREDirProvider::DoShutdown() {
   AUTO_PROFILER_LABEL("nsXREDirProvider::DoShutdown", OTHER);
 
-  if (mProfileNotified) {
+  if (mAppStarted) {
     mozilla::AppShutdown::AdvanceShutdownPhase(
         mozilla::ShutdownPhase::AppShutdownNetTeardown, nullptr);
     mozilla::AppShutdown::AdvanceShutdownPhase(
@@ -988,17 +697,11 @@ void nsXREDirProvider::DoShutdown() {
         mozilla::ShutdownPhase::AppShutdownQM, nullptr);
     mozilla::AppShutdown::AdvanceShutdownPhase(
         mozilla::ShutdownPhase::AppShutdownTelemetry, nullptr);
-    mProfileNotified = false;
+    mAppStarted = false;
   }
 
   gDataDirProfileLocal = nullptr;
   gDataDirProfile = nullptr;
-
-  if (XRE_IsParentProcess()) {
-#if defined(MOZ_SANDBOX)
-    mozilla::Unused << DeleteDirIfExists(mContentProcessSandboxTempDir);
-#endif
-  }
 }
 
 #ifdef XP_WIN
@@ -1124,7 +827,7 @@ nsresult nsXREDirProvider::GetInstallHash(nsAString& aPathHash) {
     nsCOMPtr<nsILocalFileMac> macFile = do_QueryInterface(installDir);
     rv = macFile->GetFSRef(&ref);
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = NS_NewLocalFileWithFSRef(&ref, true, getter_AddRefs(macFile));
+    rv = NS_NewLocalFileWithFSRef(&ref, getter_AddRefs(macFile));
     NS_ENSURE_SUCCESS(rv, rv);
     installDir = static_cast<nsIFile*>(macFile);
 #endif
@@ -1218,7 +921,7 @@ nsresult nsXREDirProvider::GetUpdateRootDir(nsIFile** aResult,
     return NS_ERROR_FAILURE;
   }
 
-  int32_t dotIndex = appDirPath.RFind(".app");
+  int32_t dotIndex = appDirPath.RFind(u".app");
   if (dotIndex == kNotFound) {
     dotIndex = appDirPath.Length();
   }
@@ -1261,42 +964,42 @@ nsresult nsXREDirProvider::GetUpdateRootDir(nsIFile** aResult,
   }
   nsAutoString updatePathStr;
   updatePathStr.Assign(updatePath.get());
-  updRoot->InitWithPath(updatePathStr);
-#endif  // XP_WIN
+  MOZ_TRY(updRoot->InitWithPath(updatePathStr));
   updRoot.forget(aResult);
   return NS_OK;
+#else
+  updRoot.forget(aResult);
+  return NS_OK;
+#endif  // XP_WIN
 }
 
 nsresult nsXREDirProvider::GetProfileStartupDir(nsIFile** aResult) {
-  if (mProfileDir) return mProfileDir->Clone(aResult);
-
-  if (mAppProvider) {
-    nsCOMPtr<nsIFile> needsclone;
-    bool dummy;
-    nsresult rv = mAppProvider->GetFile(NS_APP_PROFILE_DIR_STARTUP, &dummy,
-                                        getter_AddRefs(needsclone));
-    if (NS_SUCCEEDED(rv)) return needsclone->Clone(aResult);
+  if (mProfileDir) {
+    return mProfileDir->Clone(aResult);
   }
 
+  // Profile directories are only set up in the parent process.
+  // We don't expect every caller to check if they are in the right process,
+  // so fail immediately to avoid warning spam.
+  NS_WARNING_ASSERTION(!XRE_IsParentProcess(),
+                       "tried to get profile in parent too early");
   return NS_ERROR_FAILURE;
 }
 
 nsresult nsXREDirProvider::GetProfileDir(nsIFile** aResult) {
-  if (mProfileDir) {
-    if (!mProfileNotified) return NS_ERROR_FAILURE;
-
-    return mProfileDir->Clone(aResult);
+  if (!mProfileDir) {
+    nsresult rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR,
+                                         getter_AddRefs(mProfileDir));
+    // Guard against potential buggy directory providers that fail while also
+    // returning something.
+    if (NS_FAILED(rv)) {
+      MOZ_ASSERT(!mProfileDir,
+                 "Directory provider failed but returned a value");
+      mProfileDir = nullptr;
+    }
   }
-
-  if (mAppProvider) {
-    nsCOMPtr<nsIFile> needsclone;
-    bool dummy;
-    nsresult rv = mAppProvider->GetFile(NS_APP_USER_PROFILE_50_DIR, &dummy,
-                                        getter_AddRefs(needsclone));
-    if (NS_SUCCEEDED(rv)) return needsclone->Clone(aResult);
-  }
-
-  return NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, aResult);
+  // If we failed to get mProfileDir, this will warn for us if appropriate.
+  return GetProfileStartupDir(aResult);
 }
 
 NS_IMETHODIMP
@@ -1327,7 +1030,6 @@ nsresult nsXREDirProvider::SetUserDataProfileDirectory(nsCOMPtr<nsIFile>& aFile,
 nsresult nsXREDirProvider::GetUserDataDirectoryHome(nsIFile** aFile,
                                                     bool aLocal) {
   // Copied from nsAppFileLocationProvider (more or less)
-  nsresult rv;
   nsCOMPtr<nsIFile> localDir;
 
   if (aLocal && gDataDirHomeLocal) {
@@ -1352,25 +1054,20 @@ nsresult nsXREDirProvider::GetUserDataDirectoryHome(nsIFile** aFile,
   OSErr err = ::FSFindFolder(kUserDomain, folderType, kCreateFolder, &fsRef);
   NS_ENSURE_FALSE(err, NS_ERROR_FAILURE);
 
-  rv = NS_NewNativeLocalFile(""_ns, true, getter_AddRefs(localDir));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsILocalFileMac> dirFileMac = do_QueryInterface(localDir);
-  NS_ENSURE_TRUE(dirFileMac, NS_ERROR_UNEXPECTED);
-
-  rv = dirFileMac->InitWithFSRef(&fsRef);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  localDir = dirFileMac;
+  nsCOMPtr<nsILocalFileMac> dirFileMac;
+  MOZ_TRY(NS_NewLocalFileWithFSRef(&fsRef, getter_AddRefs(dirFileMac)));
+  localDir = dirFileMac.forget();
 #elif defined(XP_IOS)
   nsAutoCString userDir;
+  nsresult rv;
   if (GetUIKitDirectory(aLocal, userDir)) {
-    rv = NS_NewNativeLocalFile(userDir, true, getter_AddRefs(localDir));
+    rv = NS_NewNativeLocalFile(userDir, getter_AddRefs(localDir));
   } else {
     rv = NS_ERROR_FAILURE;
   }
   NS_ENSURE_SUCCESS(rv, rv);
 #elif defined(XP_WIN)
+  nsresult rv;
   nsString path;
   if (aLocal) {
     rv = GetShellFolderPath(FOLDERID_LocalAppData, path);
@@ -1384,7 +1081,7 @@ nsresult nsXREDirProvider::GetUserDataDirectoryHome(nsIFile** aFile,
   }
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = NS_NewLocalFile(path, true, getter_AddRefs(localDir));
+  MOZ_TRY(NS_NewLocalFile(path, getter_AddRefs(localDir)));
 #elif defined(XP_UNIX)
   const char* homeDir = getenv("HOME");
   if (!homeDir || !*homeDir) return NS_ERROR_FAILURE;
@@ -1397,23 +1094,23 @@ nsresult nsXREDirProvider::GetUserDataDirectoryHome(nsIFile** aFile,
     // If $XDG_CACHE_HOME is defined use it, otherwise use $HOME/.cache.
     const char* cacheHome = getenv("XDG_CACHE_HOME");
     if (cacheHome && *cacheHome) {
-      rv = NS_NewNativeLocalFile(nsDependentCString(cacheHome), true,
-                                 getter_AddRefs(localDir));
+      MOZ_TRY(NS_NewNativeLocalFile(nsDependentCString(cacheHome),
+                                    getter_AddRefs(localDir)));
     } else {
-      rv = NS_NewNativeLocalFile(nsDependentCString(homeDir), true,
-                                 getter_AddRefs(localDir));
-      if (NS_SUCCEEDED(rv)) rv = localDir->AppendNative(".cache"_ns);
+      MOZ_TRY(NS_NewNativeLocalFile(nsDependentCString(homeDir),
+                                    getter_AddRefs(localDir)));
+      MOZ_TRY(localDir->AppendNative(".cache"_ns));
     }
   } else {
-    rv = NS_NewNativeLocalFile(nsDependentCString(homeDir), true,
-                               getter_AddRefs(localDir));
+    MOZ_TRY(NS_NewNativeLocalFile(nsDependentCString(homeDir),
+                                  getter_AddRefs(localDir)));
   }
 #else
 #  error "Don't know how to get product dir on your platform"
 #endif
 
-  NS_IF_ADDREF(*aFile = localDir);
-  return rv;
+  localDir.forget(aFile);
+  return NS_OK;
 }
 
 nsresult nsXREDirProvider::GetSysUserExtensionsDirectory(nsIFile** aFile) {
@@ -1457,6 +1154,9 @@ nsresult nsXREDirProvider::GetSystemExtensionsDirectory(nsIFile** aFile) {
 
 nsresult nsXREDirProvider::GetUserDataDirectory(nsIFile** aFile, bool aLocal) {
   nsCOMPtr<nsIFile> localDir;
+  nsCOMPtr<nsIFile> customDir = mozilla::GetFileFromEnv("MOZ_APP_DATA");
+  nsCOMPtr<nsIFile> customLocalDir =
+      mozilla::GetFileFromEnv("MOZ_LOCAL_APP_DATA");
 
   if (aLocal && gDataDirProfileLocal) {
     return gDataDirProfileLocal->Clone(aFile);
@@ -1465,8 +1165,15 @@ nsresult nsXREDirProvider::GetUserDataDirectory(nsIFile** aFile, bool aLocal) {
     return gDataDirProfile->Clone(aFile);
   }
 
-  nsresult rv = GetUserDataDirectoryHome(getter_AddRefs(localDir), aLocal);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsresult rv;
+  if (aLocal && customLocalDir) {
+    localDir = customLocalDir;
+  } else if (!aLocal && customDir) {
+    localDir = customDir;
+  } else {
+    rv = GetUserDataDirectoryHome(getter_AddRefs(localDir), aLocal);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
 
   rv = AppendProfilePath(localDir, aLocal);
   NS_ENSURE_SUCCESS(rv, rv);
