@@ -4,38 +4,40 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+import glob
 import itertools
 import json
-import glob
 import os
 import re
-import six
+import shlex
 import subprocess
 import sys
-
 import xml.etree.ElementTree as ET
 
-from mozpack.files import FileFinder
 import mozpack.path as mozpath
 from mozlint import result
+from mozpack.files import FileFinder
 
 # The Gradle target invocations are serialized with a simple locking file scheme.  It's fine for
 # them to take a while, since the first will compile all the Java, etc, and then perform
 # potentially expensive static analyses.
 GRADLE_LOCK_MAX_WAIT_SECONDS = 20 * 60
 
+EXCLUSION_FILES = [
+    os.path.join("tools", "rewriting", "Generated.txt"),
+    os.path.join("tools", "rewriting", "ThirdPartyPaths.txt"),
+]
+
 
 def setup(root, **setupargs):
     if setupargs.get("substs", {}).get("MOZ_BUILD_APP") != "mobile/android":
-        return 1
+        return -1
 
     if "topobjdir" not in setupargs:
-        print(
-            "Skipping {}: a configured Android build is required!".format(
-                setupargs["name"]
-            )
+        setupargs["log"].debug(
+            f"Skipping {setupargs['name']}: a configured Android build is required!"
         )
-        return 1
+        return -1
 
     return 0
 
@@ -61,7 +63,7 @@ def gradle(log, topsrcdir=None, topobjdir=None, tasks=[], extra_args=[], verbose
             + extra_args
         )
 
-        cmd = " ".join(six.moves.shlex_quote(arg) for arg in cmd_args)
+        cmd = " ".join(shlex.quote(arg) for arg in cmd_args)
         log.debug(cmd)
 
         # Gradle and mozprocess do not get along well, so we use subprocess
@@ -83,8 +85,10 @@ def gradle(log, topsrcdir=None, topobjdir=None, tasks=[], extra_args=[], verbose
             proc.kill()
             raise
 
+        return proc.returncode
 
-def format(config, fix=None, **lintargs):
+
+def format(_paths, config, fix=None, **lintargs):
     topsrcdir = lintargs["root"]
     topobjdir = lintargs["topobjdir"]
 
@@ -93,7 +97,7 @@ def format(config, fix=None, **lintargs):
     else:
         tasks = lintargs["substs"]["GRADLE_ANDROID_FORMAT_LINT_CHECK_TASKS"]
 
-    gradle(
+    ret = gradle(
         lintargs["log"],
         topsrcdir=topsrcdir,
         topobjdir=topobjdir,
@@ -103,19 +107,49 @@ def format(config, fix=None, **lintargs):
 
     results = []
     for path in lintargs["substs"]["GRADLE_ANDROID_FORMAT_LINT_FOLDERS"]:
-        folder = os.path.join(
-            topobjdir, "gradle", "build", path, "spotless", "spotlessJava"
-        )
-        for filename in glob.iglob(folder + "/**/*.java", recursive=True):
-            err = {
-                "rule": "spotless-java",
-                "path": os.path.join(path, mozpath.relpath(filename, folder)),
-                "lineno": 0,
-                "column": 0,
-                "message": "Formatting error, please run ./mach lint -l android-format --fix",
-                "level": "error",
-            }
-            results.append(result.from_config(config, **err))
+        for outdir in ("spotless-clean", "spotless-lints"):
+            folder = os.path.join(
+                topobjdir, "gradle", "build", path, outdir, "spotlessJava"
+            )
+            for filename in glob.iglob(folder + "/**/*.java", recursive=True):
+                err = {
+                    "rule": "spotless-java",
+                    "path": os.path.join(
+                        topsrcdir, path, mozpath.relpath(filename, folder)
+                    ),
+                    "lineno": 0,
+                    "column": 0,
+                    "message": "Formatting error, please run ./mach lint -l android-format --fix",
+                    "level": "error",
+                }
+                results.append(result.from_config(config, **err))
+            folder = os.path.join(
+                topobjdir, "gradle", "build", path, outdir, "spotlessKotlin"
+            )
+            for filename in glob.iglob(folder + "/**/*.kt", recursive=True):
+                err = {
+                    "rule": "spotless-kt",
+                    "path": os.path.join(
+                        topsrcdir, path, mozpath.relpath(filename, folder)
+                    ),
+                    "lineno": 0,
+                    "column": 0,
+                    "message": "Formatting error, please run ./mach lint -l android-format --fix",
+                    "level": "error",
+                }
+                results.append(result.from_config(config, **err))
+
+    if len(results) == 0 and ret != 0:
+        # spotless seems to hit unfixed error.
+        err = {
+            "rule": "spotless",
+            "path": "",
+            "lineno": 0,
+            "column": 0,
+            "message": "Unexpected error",
+            "level": "error",
+        }
+        results.append(result.from_config(config, **err))
 
     # If --fix was passed, we just report the number of files that were changed
     if fix:
@@ -123,7 +157,148 @@ def format(config, fix=None, **lintargs):
     return results
 
 
-def api_lint(config, **lintargs):
+def fenix_format(_paths, config, fix=None, **lintargs):
+    return report_gradlew(
+        config,
+        fix,
+        os.path.join("mobile", "android", "fenix"),
+        lint_tasks=[
+            "fenix:lint",
+            "fenix:lintFenixDebug",
+        ],
+        **lintargs,
+    )
+
+
+def ac_format(_paths, config, fix=None, **lintargs):
+    return report_gradlew(
+        config,
+        fix,
+        os.path.join("mobile", "android", "android-components"),
+        lint_tasks=["lint-a-c"],
+        **lintargs,
+    )
+
+
+def focus_format(_paths, config, fix=None, **lintargs):
+    return report_gradlew(
+        config,
+        fix,
+        os.path.join("mobile", "android", "focus-android"),
+        lint_tasks=["focus-android:lint"],
+        **lintargs,
+    )
+
+
+def report_gradlew(config, fix, subdir, lint_tasks=[], **lintargs):
+    topsrcdir = lintargs["root"]
+    topobjdir = lintargs["topobjdir"]
+
+    if fix:
+        tasks = ["ktlintFormat", "detekt"]
+    else:
+        tasks = ["ktlint", "detekt"]
+
+    extra_args = lintargs.get("extra_args") or []
+
+    gradle(
+        lintargs["log"],
+        topsrcdir=topsrcdir,
+        topobjdir=topobjdir,
+        tasks=tasks,
+        extra_args=extra_args + ["-p", os.path.join(topsrcdir, subdir), "--continue"],
+    )
+
+    reports = os.path.join(topsrcdir, subdir, "build", "reports")
+    results = []
+
+    excludes = []
+    for path in EXCLUSION_FILES:
+        with open(os.path.join(topsrcdir, path)) as fh:
+            for f in fh.readlines():
+                if "*" in f:
+                    excludes.extend(glob.glob(f.strip()))
+                elif f.startswith(subdir):
+                    excludes.append(f.strip())
+
+    detekt_report = None
+    if os.path.exists(os.path.join(reports, "detekt", "detekt.xml")):
+        detekt_report = os.path.join(reports, "detekt", "detekt.xml")
+    elif os.path.join(
+        topobjdir, "gradle", "build", subdir, "reports", "detekt", "detekt.xml"
+    ):
+        detekt_report = os.path.join(
+            topobjdir, "gradle", "build", subdir, "reports", "detekt", "detekt.xml"
+        )
+    else:
+        print(f"Could not read detekt report: '{detekt_report}'")
+        pass
+
+    try:
+        tree = ET.parse(open(detekt_report))
+        root = tree.getroot()
+
+        for file in root.findall("file"):
+            name = file.get("name")
+            if is_excluded_file(topsrcdir, excludes, name):
+                continue
+            for error in file:
+                err = {
+                    "rule": error.get("source"),
+                    "path": name,
+                    "lineno": int(error.get("line") or 0),
+                    "column": int(error.get("column") or 0),
+                    "message": error.get("message"),
+                    "level": "error",
+                }
+                results.append(result.from_config(config, **err))
+    except FileNotFoundError:
+        print(f"Could not read detekt report: '{detekt_report}'")
+        pass
+
+    ktlint_file = "ktlint.json"
+    if fix:
+        ktlint_file = "ktlintFormat.json"
+    try:
+        issues = json.load(
+            open(
+                os.path.join(
+                    reports,
+                    "ktlint",
+                    ktlint_file,
+                ),
+            )
+        )
+
+        for issue in issues:
+            name = issue["file"]
+            if is_excluded_file(topsrcdir, excludes, name):
+                continue
+            for error in issue["errors"]:
+                err = {
+                    "rule": error["rule"],
+                    "path": name,
+                    "lineno": error["line"],
+                    "column": error["column"],
+                    "message": error["message"],
+                    "level": "error",
+                }
+                results.append(result.from_config(config, **err))
+    except FileNotFoundError:
+        print(f"Could not read ktlint report: `{ktlint_file}`")
+        pass
+
+    return results + read_lint_report(config, subdir, tasks=lint_tasks, **lintargs)
+
+
+def is_excluded_file(topsrcdir, excludes, file):
+    for path in excludes:
+        if file.startswith(os.path.join(topsrcdir, path)):
+            return True
+    return False
+
+
+def api_lint(_paths, config, **lintargs):
     topsrcdir = lintargs["root"]
     topobjdir = lintargs["topobjdir"]
 
@@ -146,7 +321,7 @@ def api_lint(config, **lintargs):
             for r in issues[rule]:
                 err = {
                     "rule": r["rule"] if rule == "failures" else "compat_failures",
-                    "path": mozpath.relpath(r["file"], topsrcdir),
+                    "path": r["file"],
                     "lineno": int(r["line"]),
                     "column": int(r.get("column") or 0),
                     "message": r["msg"],
@@ -157,7 +332,7 @@ def api_lint(config, **lintargs):
         for r in issues["api_changes"]:
             err = {
                 "rule": "api_changes",
-                "path": mozpath.relpath(r["file"], topsrcdir),
+                "path": r["file"],
                 "lineno": int(r["line"]),
                 "column": int(r.get("column") or 0),
                 "message": "Unexpected api change. Please run ./mach gradle {} for more "
@@ -170,7 +345,7 @@ def api_lint(config, **lintargs):
     return results
 
 
-def javadoc(config, **lintargs):
+def javadoc(_paths, config, **lintargs):
     topsrcdir = lintargs["root"]
     topobjdir = lintargs["topobjdir"]
 
@@ -192,7 +367,6 @@ def javadoc(config, **lintargs):
             issues = json.load(f)
 
             for issue in issues:
-                issue["path"] = issue["path"].replace(lintargs["root"], "")
                 # We want warnings to be errors for linting purposes.
                 # TODO: Bug 1316188 - resolve missing javadoc comments
                 issue["level"] = (
@@ -203,7 +377,7 @@ def javadoc(config, **lintargs):
     return results
 
 
-def lint(config, **lintargs):
+def lint(_paths, config, **lintargs):
     topsrcdir = lintargs["root"]
     topobjdir = lintargs["topobjdir"]
 
@@ -215,9 +389,6 @@ def lint(config, **lintargs):
         extra_args=lintargs.get("extra_args") or [],
     )
 
-    # It's surprising that this is the App variant name, but this is "withoutGeckoBinariesDebug"
-    # right now and the GeckoView variant name is "withGeckoBinariesDebug".  This will be addressed
-    # as we unify variants.
     path = os.path.join(
         lintargs["topobjdir"],
         "gradle/build/mobile/android/geckoview/reports",
@@ -225,7 +396,7 @@ def lint(config, **lintargs):
             lintargs["substs"]["GRADLE_ANDROID_GECKOVIEW_VARIANT_NAME"]
         ),
     )
-    tree = ET.parse(open(path, "rt"))
+    tree = ET.parse(open(path))
     root = tree.getroot()
 
     results = []
@@ -240,7 +411,7 @@ def lint(config, **lintargs):
             "level": issue.get("severity").lower(),
             "rule": issue.get("id"),
             "message": issue.get("message"),
-            "path": location.get("file").replace(lintargs["root"], ""),
+            "path": location.get("file"),
             "lineno": int(location.get("line") or 0),
         }
         results.append(result.from_config(config, **err))
@@ -248,27 +419,127 @@ def lint(config, **lintargs):
     return results
 
 
+def read_lint_report(config, subdir, tasks=[], **lintargs):
+    topsrcdir = lintargs["root"]
+    topobjdir = lintargs["topobjdir"]
+
+    ret = gradle(
+        lintargs["log"],
+        topsrcdir=topsrcdir,
+        topobjdir=topobjdir,
+        tasks=tasks,
+        extra_args=lintargs.get("extra_args") or [],
+    )
+
+    reports = os.path.join(topsrcdir, subdir, "build", "reports")
+
+    excludes = []
+    for path in EXCLUSION_FILES:
+        with open(os.path.join(topsrcdir, path)) as fh:
+            for f in fh.readlines():
+                if "*" in f:
+                    excludes.extend(glob.glob(f.strip()))
+                elif f.startswith(subdir):
+                    excludes.append(f.strip())
+
+    try:
+        files = os.listdir(
+            os.path.join(
+                reports,
+                "lint",
+            )
+        )
+
+        results = []
+        for file in files:
+            data = json.load(
+                open(
+                    os.path.join(reports, "lint", file),
+                )
+            ).get(
+                "runs", [{}]
+            )[0]
+
+            issues = data.get("results", [])
+            rules = data.get("tool", {}).get("driver", {}).get("rules", [])
+
+            for issue in issues:
+                dir = os.path.join(topsrcdir)
+                if subdir == os.path.join("mobile", "android", "android-components"):
+                    dir = os.path.join(topsrcdir, subdir)
+                name = os.path.join(
+                    dir,
+                    issue.get("locations", [{}])[0]
+                    .get("physicalLocation", {})
+                    .get("artifactLocation", {})
+                    .get("uri"),
+                )
+
+                if is_excluded_file(topsrcdir, excludes, name) and not "/res/" in name:
+                    continue
+
+                level = "error"
+                if "level" in issue:
+                    level = issue["level"]
+                elif "ruleIndex" in issue and len(rules) > issue["ruleIndex"]:
+                    rule_level = (
+                        rules[issue["ruleIndex"]]
+                        .get("defaultConfiguration", {})
+                        .get("level")
+                    )
+                    if rule_level:
+                        level = rule_level
+
+                err = {
+                    "rule": issue.get("ruleId"),
+                    "path": name,
+                    "lineno": issue.get("locations", [{}])[0]
+                    .get("physicalLocation", {})
+                    .get("region", {})
+                    .get("startLine"),
+                    "column": issue.get("locations", [{}])[0]
+                    .get("physicalLocation", {})
+                    .get("region", {})
+                    .get("startColumn"),
+                    "message": issue.get("message", {}).get("text"),
+                    "level": level,
+                }
+                results.append(result.from_config(config, **err))
+
+        if ret != 0 and results == []:
+            err = {
+                "level": "error",
+                "rule": "build-failure",
+                "message": f"Build Failed running {tasks} - Please check logs for more information",
+                "path": os.path.join(topsrcdir, subdir),
+                "lineno": 0,
+            }
+            results.append(result.from_config(config, **err))
+        return results
+    except FileNotFoundError:
+        print("Could not read lint report from ", subdir)
+        return []
+
+
 def _parse_checkstyle_output(config, topsrcdir=None, report_path=None):
-    tree = ET.parse(open(report_path, "rt"))
+    tree = ET.parse(open(report_path))
     root = tree.getroot()
 
     for file in root.findall("file"):
-        sourcepath = file.get("name").replace(topsrcdir + "/", "")
-
         for error in file.findall("error"):
             # Like <error column="42" line="22" message="Name 'mPorts' must match pattern 'xm[A-Z][A-Za-z]*$'." severity="error" source="com.puppycrawl.tools.checkstyle.checks.naming.MemberNameCheck" />.  # NOQA: E501
             err = {
                 "level": "error",
                 "rule": error.get("source"),
                 "message": error.get("message"),
-                "path": sourcepath,
+                "path": file.get("name"),
                 "lineno": int(error.get("line") or 0),
                 "column": int(error.get("column") or 0),
             }
             yield result.from_config(config, **err)
 
 
-def checkstyle(config, **lintargs):
+def checkstyle(_paths, config, **lintargs):
     topsrcdir = lintargs["root"]
     topobjdir = lintargs["topobjdir"]
 
@@ -301,10 +572,10 @@ def _parse_android_test_results(config, topsrcdir=None, report_dir=None):
     finder = FileFinder(report_dir)
     reports = list(finder.find("TEST-*.xml"))
     if not reports:
-        raise RuntimeError("No reports found under {}".format(report_dir))
+        raise RuntimeError(f"No reports found under {report_dir}")
 
     for report, _ in reports:
-        tree = ET.parse(open(os.path.join(finder.base, report), "rt"))
+        tree = ET.parse(open(os.path.join(finder.base, report)))
         root = tree.getroot()
 
         class_name = root.get(
@@ -323,11 +594,7 @@ def _parse_android_test_results(config, topsrcdir=None, report_dir=None):
             ):
                 sourcepaths = list(sourcepath_finder.find(path))
                 if not sourcepaths:
-                    raise RuntimeError(
-                        "No sourcepath found for class {class_name}".format(
-                            class_name=class_name
-                        )
-                    )
+                    raise RuntimeError(f"No sourcepath found for class {class_name}")
 
                 for sourcepath, _ in sourcepaths:
                     lineno = 0
@@ -341,22 +608,22 @@ def _parse_android_test_results(config, topsrcdir=None, report_dir=None):
                     if match:
                         lineno = int(match.group(1))
                     else:
-                        msg = "No source line found for {class_name}.{function_name}".format(
-                            class_name=class_name, function_name=function_name
-                        )
+                        msg = f"No source line found for {class_name}.{function_name}"
                         raise RuntimeError(msg)
 
                     err = {
                         "level": "error",
                         "rule": unexpected.get("type"),
                         "message": message,
-                        "path": os.path.join("mobile", "android", sourcepath),
+                        "path": os.path.join(
+                            topsrcdir, "mobile", "android", sourcepath
+                        ),
                         "lineno": lineno,
                     }
                     yield result.from_config(config, **err)
 
 
-def test(config, **lintargs):
+def test(_paths, config, **lintargs):
     topsrcdir = lintargs["root"]
     topobjdir = lintargs["topobjdir"]
 
@@ -378,9 +645,7 @@ def test(config, **lintargs):
     for project, variant in pairs:
         report_dir = os.path.join(
             lintargs["topobjdir"],
-            "gradle/build/mobile/android/{}/test-results/test{}UnitTest".format(
-                project, capitalize(variant)
-            ),
+            f"gradle/build/mobile/android/{project}/test-results/test{capitalize(variant)}UnitTest",
         )
     results.extend(
         _parse_android_test_results(
