@@ -1,5 +1,5 @@
 /**
- * @fileoverview A processor to help parse the spidermonkey js code.
+ * @file A processor to help parse the spidermonkey js code.
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
@@ -11,7 +11,11 @@
 // Plugin Definition
 // ------------------------------------------------------------------------------
 module.exports = {
+  meta: { name: "eslint-plugin-spidermonkey-js", version: "0.1.1" },
   processors: {
-    ".js": require("../lib/processors/self-hosted"),
+    processor: require("./processors/self-hosted"),
+  },
+  environments: {
+    environment: require("./environments/self-hosted"),
   },
 };
