@@ -1,19 +1,18 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from __future__ import absolute_import
-
 import json
 import os
 import pathlib
 import re
 
+from gecko_taskgraph.util.attributes import match_run_on_projects
 from manifestparser import TestManifest
 from mozperftest.script import ScriptInfo
-from perfdocs.utils import read_yaml
-from perfdocs.logger import PerfDocLogger
+
 from perfdocs.doc_helpers import TableBuilder
-from gecko_taskgraph.util.attributes import match_run_on_projects
+from perfdocs.logger import PerfDocLogger
+from perfdocs.utils import read_yaml
 
 logger = PerfDocLogger()
 
@@ -31,7 +30,7 @@ must implement the FrameworkGatherer class.
 """
 
 
-class FrameworkGatherer(object):
+class FrameworkGatherer:
     """
     Abstract class for framework gatherers.
     """
@@ -52,6 +51,38 @@ class FrameworkGatherer(object):
         self._task_list = {}
         self._task_match_pattern = re.compile(r"([\w\W]*/[pgo|opt]*)-([\w\W]*)")
 
+    def _build_section_with_header(self, title, content, header_type=None):
+        """
+        Adds a section to the documentation with the title as the type mentioned
+        and paragraph as content mentioned.
+        :param title: title of the section
+        :param content: content of section paragraph
+        :param header_type: type of the title heading
+        """
+        heading_map = {"H2": "*", "H3": "=", "H4": "-", "H5": "^"}
+        return [title, heading_map.get(header_type, "^") * len(title), content, ""]
+
+    def _get_metric_heading(self, metric, metrics_info):
+        """
+        Gets the heading of a specific metric.
+
+        :param str metric: The metric to search for.
+        :param dict metrics_info: The information of all the
+            metrics that were documented.
+        :return str: The heading to use for the given metric.
+        """
+        for metric_heading, metric_info in metrics_info.items():
+            if metric == metric_heading or any(
+                metric == alias for alias in metric_info.get("aliases", [])
+            ):
+                return metric_heading
+            if metric_info.get("matcher"):
+                match = re.search(metric_info["matcher"], metric)
+                if match:
+                    return metric_heading
+
+        raise Exception(f"Could not find a metric heading for `{metric}`")
+
     def get_task_match(self, task_name):
         return re.search(self._task_match_pattern, task_name)
 
@@ -67,7 +98,7 @@ class FrameworkGatherer(object):
             return self._manifest_path
 
         yaml_content = read_yaml(self._yaml_path)
-        self._manifest_path = os.path.join(self.workspace_dir, yaml_content["manifest"])
+        self._manifest_path = pathlib.Path(self.workspace_dir, yaml_content["manifest"])
         return self._manifest_path
 
     def get_suite_list(self):
@@ -86,16 +117,41 @@ class FrameworkGatherer(object):
         """
         raise NotImplementedError
 
-    def _build_section_with_header(self, title, content, header_type=None):
+    def build_metrics_documentation(self, yaml_content):
         """
-        Adds a section to the documentation with the title as the type mentioned
-        and paragraph as content mentioned.
-        :param title: title of the section
-        :param content: content of section paragraph
-        :param header_type: type of the title heading
+        Each framework that provides a page with descriptions about the
+        metrics it produces must implement this method. The metrics defined
+        for the framework can be found in the `yaml_content` variable.
+
+        The framework gatherer is expected to produce the full documentation
+        for all the metrics defined in the yaml_content at once. This is done
+        to allow differentiation between how metrics are displayed between
+        the different frameworks.
+
+        :param dict yaml_content: A dictionary of the YAML config file for
+            the specific framework.
+        :return list: A list of all the lines being added to the metrics
+            documentation.
         """
-        heading_map = {"H2": "*", "H3": "=", "H4": "-", "H5": "^"}
-        return [title, heading_map.get(header_type, "^") * len(title), content, ""]
+        raise NotImplementedError
+
+    def build_command_to_run_locally(self, framework_command, title):
+        """
+        Each framework has specifics to running it locally. This command
+        passes arguments to this function to ensure we can construct those
+        commands consistently, and return it so it can be in the mozilla source docs.
+
+        :param str framework_command: A string that has the framework specific
+            commands needed to run tests
+        :param str title: A string of the test name, added on after the framework
+            specific commands (see above framework_command param:)
+        :return str: Returns the command to run locally, this output is added to
+            the mozilla source docs, and is formatted
+        """
+        command_to_run_locally = "   * Command to Run Locally\n\n"
+        command_to_run_locally += "   .. code-block::\n\n"
+        command_to_run_locally += f"      ./mach {framework_command} {title}\n\n"
+        return command_to_run_locally
 
 
 class RaptorGatherer(FrameworkGatherer):
@@ -121,7 +177,7 @@ class RaptorGatherer(FrameworkGatherer):
         manifest_path = self.get_manifest_path()
 
         # Get the tests from the manifest
-        test_manifest = TestManifest([manifest_path], strict=False)
+        test_manifest = TestManifest([str(manifest_path)], strict=False)
         test_list = test_manifest.active_tests(exists=False, disabled=False)
 
         # Parse the tests into the expected dictionary
@@ -141,7 +197,7 @@ class RaptorGatherer(FrameworkGatherer):
 
     def _get_ci_tasks(self):
         for task in self._taskgraph.keys():
-            if type(self._taskgraph[task]) == dict:
+            if type(self._taskgraph[task]) is dict:
                 command = self._taskgraph[task]["task"]["payload"].get("command", [])
                 run_on_projects = self._taskgraph[task]["attributes"]["run_on_projects"]
             else:
@@ -167,8 +223,10 @@ class RaptorGatherer(FrameworkGatherer):
         :param str manifest_path: path to the ini file
         :return list: the list of the tests
         """
-        desc_exclusion = ["here", "manifest", "manifest_relpath", "path", "relpath"]
-        test_manifest = TestManifest([manifest_path], strict=False)
+        desc_exclusion = ["here", "manifest_relpath", "path", "relpath"]
+        test_manifest = TestManifest(
+            [str(manifest_path)], strict=False, document=True, add_line_no=True
+        )
         test_list = test_manifest.active_tests(exists=False, disabled=False)
         subtests = {}
         for subtest in test_list:
@@ -178,12 +236,56 @@ class RaptorGatherer(FrameworkGatherer):
             for key, value in subtest.items():
                 if key not in desc_exclusion:
                     description[key] = value
+
+            # Add searchfox link
+            key = list(test_manifest.source_documents.keys())[0]
+
+            if (
+                test_manifest.source_documents[key]
+                and subtest["name"] in test_manifest.source_documents[key].keys()
+            ):
+                description["link searchfox"] = (
+                    "https://searchfox.org/mozilla-central/source/"
+                    + pathlib.Path(manifest_path).as_posix()
+                    + "#"
+                    + test_manifest.source_documents[key][subtest["name"]]["lineno"]
+                )
+
+            # Prepare alerting metrics for verification
+            description["metrics"] = [
+                metric.strip()
+                for metric in description.get("alert_on", "").split(",")
+                if metric.strip() != ""
+            ]
+            if (
+                description.get("gather_cpuTime", None)
+                or "cpuTime" in description.get("measure", [])
+                or suite_name in ["desktop", "interactive", "mobile"]
+            ):
+                description["metrics"].append("cpuTime")
+
+            subtests[subtest["name"]] = description
             self._descriptions.setdefault(suite_name, []).append(description)
 
         self._descriptions[suite_name].sort(key=lambda item: item["name"])
 
         return subtests
 
+    def _get_metric_heading(self, metric, metrics_info):
+        """
+        Finds, and returns the correct heading for a metric to target in a reference link.
+
+        :param str metric: The metric to search for.
+        :param dict metrics_info: The information of all the
+            metrics that were documented.
+        :return str: A formatted string containing the reference link to the
+            documented metric.
+        """
+        metric_heading = super(RaptorGatherer, self)._get_metric_heading(
+            metric, metrics_info
+        )
+        return f"`{metric} <raptor-metrics.html#{metric_heading.lower().replace(' ', '-')}>`__"
+
     def get_test_list(self):
         """
         Returns a dictionary containing the tests in every suite ini file.
@@ -213,12 +315,13 @@ class RaptorGatherer(FrameworkGatherer):
 
         return self._test_list
 
-    def build_test_description(self, title, test_description="", suite_name=""):
+    def build_test_description(
+        self, title, test_description="", suite_name="", metrics_info=None
+    ):
         matcher = []
         browsers = [
             "firefox",
             "chrome",
-            "chromium",
             "refbrow",
             "fennec68",
             "geckoview",
@@ -235,38 +338,55 @@ class RaptorGatherer(FrameworkGatherer):
         if len(matcher) == 0:
             logger.critical(
                 "No tests exist for the following name "
-                "(obtained from config.yml): {}".format(title)
+                f"(obtained from config.yml): {title}"
             )
             raise Exception(
                 "No tests exist for the following name "
-                "(obtained from config.yml): {}".format(title)
+                f"(obtained from config.yml): {title}"
             )
 
         result = f".. dropdown:: {title}\n"
-        result += f"   :container: + anchor-id-{title}-{suite_name[0]}\n\n"
+        result += f"   :class-container: anchor-id-{title}-{suite_name[0]}\n\n"
+        result += self.build_command_to_run_locally("raptor -t", title)
 
         for idx, description in enumerate(matcher):
             if description["name"] != title:
                 result += f"   {idx+1}. **{description['name']}**\n\n"
             if "owner" in description.keys():
                 result += f"   **Owner**: {description['owner']}\n\n"
+            if test_description:
+                result += f"   **Description**: {test_description}\n\n"
 
             for key in sorted(description.keys()):
-                if key in ["owner", "name"]:
+                if key in ["owner", "name", "manifest", "metrics"]:
                     continue
                 sub_title = key.replace("_", " ")
                 if key == "test_url":
                     if "<" in description[key] or ">" in description[key]:
-                        description[key] = description[key].replace("<", "\<")
-                        description[key] = description[key].replace(">", "\>")
+                        description[key] = description[key].replace("<", r"\<")
+                        description[key] = description[key].replace(">", r"\>")
                     result += f"   * **{sub_title}**: `<{description[key]}>`__\n"
                 elif key == "secondary_url":
                     result += f"   * **{sub_title}**: `<{description[key]}>`__\n"
+                elif key == "link searchfox":
+                    result += f"   * **{sub_title}**: `<{description[key]}>`__\n"
                 elif key in ["playback_pageset_manifest"]:
                     result += (
                         f"   * **{sub_title}**: "
                         f"{description[key].replace('{subtest}', description['name'])}\n"
                     )
+                elif key == "alert_on":
+                    result += (
+                        f"   * **{sub_title}**: "
+                        + ", ".join(
+                            self._get_metric_heading(metric.strip(), metrics_info)
+                            for metric in description[key]
+                            .replace("\n", " ")
+                            .replace(",", " ")
+                            .split()
+                        )
+                        + "\n"
+                    )
                 else:
                     if "\n" in description[key]:
                         description[key] = description[key].replace("\n", " ")
@@ -275,6 +395,10 @@ class RaptorGatherer(FrameworkGatherer):
             if self._task_list.get(title, []):
                 result += "   * **Test Task**:\n\n"
                 for platform in sorted(self._task_list[title]):
+                    if (suite_name == "mobile" and "android" not in platform) or (
+                        suite_name == "desktop" and "android" in platform
+                    ):
+                        continue
                     self._task_list[title][platform].sort(key=lambda x: x["test_name"])
 
                     table = TableBuilder(
@@ -288,9 +412,11 @@ class RaptorGatherer(FrameworkGatherer):
                     for task in self._task_list[title][platform]:
                         values = [task["test_name"]]
                         values += [
-                            "\u2705"
-                            if match_run_on_projects(x, task["run_on_projects"])
-                            else "\u274C"
+                            (
+                                "\u2705"
+                                if match_run_on_projects(x, task["run_on_projects"])
+                                else "\u274C"
+                            )
                             for x in BRANCHES
                         ]
                         table.add_row(values)
@@ -303,6 +429,40 @@ class RaptorGatherer(FrameworkGatherer):
             title.capitalize(), content, header_type="H4"
         )
 
+    def build_metrics_documentation(self, parsed_metrics):
+        metrics_documentation = []
+        for metric, metric_info in sorted(
+            parsed_metrics.items(), key=lambda item: item[0]
+        ):
+            metric_content = metric_info["description"] + "\n\n"
+
+            metric_content += (
+                f"  * **Aliases**: {', '.join(sorted(metric_info['aliases']))}\n"
+            )
+            if metric_info.get("location", None):
+                metric_content += "  * **Tests using it**:\n"
+
+                for suite, tests in sorted(
+                    metric_info["location"].items(), key=lambda item: item[0]
+                ):
+                    metric_content += f"     * **{suite.capitalize()}**: "
+
+                    test_links = []
+                    for test in sorted(tests):
+                        test_links.append(
+                            f"`{test} <raptor.html#{test}-{suite.lower()[0]}>`__"
+                        )
+
+                    metric_content += ", ".join(test_links) + "\n"
+
+            metrics_documentation.extend(
+                self._build_section_with_header(
+                    metric, metric_content, header_type="H3"
+                )
+            )
+
+        return metrics_documentation
+
 
 class MozperftestGatherer(FrameworkGatherer):
     """
@@ -311,7 +471,7 @@ class MozperftestGatherer(FrameworkGatherer):
 
     def get_test_list(self):
         """
-        Returns a dictionary containing the tests that are in perftest.ini manifest.
+        Returns a dictionary containing the tests that are in perftest.toml manifest.
 
         :return dict: A dictionary with the following structure: {
                 "suite_name": {
@@ -320,10 +480,10 @@ class MozperftestGatherer(FrameworkGatherer):
                 },
             }
         """
-        for path in pathlib.Path(self.workspace_dir).rglob("perftest.ini"):
-            if "obj-" in str(path):
+        for path in list(pathlib.Path(self.workspace_dir).rglob("perftest.toml")):
+            if "obj-" in str(path) or "objdir-" in str(path):
                 continue
-            suite_name = re.sub(self.workspace_dir, "", os.path.dirname(path))
+            suite_name = str(path.parent).replace(str(self.workspace_dir), "")
 
             # If the workspace dir doesn't end with a forward-slash,
             # the substitution above won't work completely
@@ -334,19 +494,26 @@ class MozperftestGatherer(FrameworkGatherer):
             # because mozperftest tests exist in multiple places in-tree
             PerfDocLogger.PATHS.append(suite_name)
 
-            # Get the tests from perftest.ini
+            # Get the tests from perftest.toml
             test_manifest = TestManifest([str(path)], strict=False)
-            test_list = test_manifest.active_tests(exists=False, disabled=False)
+            test_list = test_manifest.active_tests(exists=False, disabled=True)
             for test in test_list:
                 si = ScriptInfo(test["path"])
-                self.script_infos[si["name"]] = si
+                if si["name"].endswith(".js"):
+                    cleaned_name = si["name"]
+                else:
+                    cleaned_name = si["name"].replace(".", "")
+
+                self.script_infos[cleaned_name] = si
                 self._test_list.setdefault(suite_name.replace("\\", "/"), {}).update(
-                    {si["name"]: str(path)}
+                    {cleaned_name: {"path": str(path)}}
                 )
 
         return self._test_list
 
-    def build_test_description(self, title, test_description="", suite_name=""):
+    def build_test_description(
+        self, title, test_description="", suite_name="", metrics_info=None
+    ):
         return [str(self.script_infos[title])]
 
     def build_suite_section(self, title, content):
@@ -356,14 +523,14 @@ class MozperftestGatherer(FrameworkGatherer):
 class TalosGatherer(FrameworkGatherer):
     def _get_ci_tasks(self):
         with open(
-            os.path.join(self.workspace_dir, "testing", "talos", "talos.json")
+            pathlib.Path(self.workspace_dir, "testing", "talos", "talos.json")
         ) as f:
             config_suites = json.load(f)["suites"]
 
         for task_name in self._taskgraph.keys():
             task = self._taskgraph[task_name]
 
-            if type(task) == dict:
+            if type(task) is dict:
                 is_talos = task["task"]["extra"].get("suite", [])
                 command = task["task"]["payload"].get("command", [])
                 run_on_projects = task["attributes"]["run_on_projects"]
@@ -394,7 +561,7 @@ class TalosGatherer(FrameworkGatherer):
         suite_name = "Talos Tests"
 
         for test in test_lists:
-            self._test_list.setdefault(suite_name, {}).update({test: ""})
+            self._test_list.setdefault(suite_name, {}).update({test: {}})
 
             klass = getattr(mod, test)
             self._descriptions.setdefault(test, klass.__dict__)
@@ -403,9 +570,12 @@ class TalosGatherer(FrameworkGatherer):
 
         return self._test_list
 
-    def build_test_description(self, title, test_description="", suite_name=""):
+    def build_test_description(
+        self, title, test_description="", suite_name="", metrics_info=None
+    ):
         result = f".. dropdown:: {title}\n"
-        result += f"   :container: + anchor-id-{title}\n\n"
+        result += f"   :class-container: anchor-id-{title}\n\n"
+        result += self.build_command_to_run_locally("talos-test -a", title)
 
         yml_descriptions = [s.strip() for s in test_description.split("- ") if s]
         for description in yml_descriptions:
@@ -413,9 +583,10 @@ class TalosGatherer(FrameworkGatherer):
                 # Example Data for using code block
                 example_list = [s.strip() for s in description.split("* ")]
                 result += f"   * {example_list[0]}\n"
-                result += "   .. code-block:: None\n\n"
+                result += "\n   .. code-block::\n\n"
                 for example in example_list[1:]:
                     result += f"      {example}\n"
+                result += "\n"
 
             elif "    * " in description:
                 # Sub List
@@ -435,7 +606,13 @@ class TalosGatherer(FrameworkGatherer):
                 elif key == "filters":
                     continue
 
-                result += f"   * {key}: {self._descriptions[title][key]}\n"
+                # On windows, we get the paths in the wrong style
+                value = self._descriptions[title][key]
+                if isinstance(value, dict):
+                    for k, v in value.items():
+                        if isinstance(v, str) and "\\" in v:
+                            value[k] = str(v).replace("\\", r"/")
+                result += r"   * " + key + r": " + str(value) + r"\n"
 
         if self._task_list.get(title, []):
             result += "   * **Test Task**:\n\n"
@@ -453,9 +630,11 @@ class TalosGatherer(FrameworkGatherer):
                 for task in self._task_list[title][platform]:
                     values = [task["test_name"]]
                     values += [
-                        "\u2705"
-                        if match_run_on_projects(x, task["run_on_projects"])
-                        else "\u274C"
+                        (
+                            "\u2705"
+                            if match_run_on_projects(x, task["run_on_projects"])
+                            else "\u274C"
+                        )
                         for x in BRANCHES
                     ]
                     table.add_row(values)
@@ -476,7 +655,7 @@ class AwsyGatherer(FrameworkGatherer):
         for task_name in self._taskgraph.keys():
             task = self._taskgraph[task_name]
 
-            if type(task) == dict:
+            if type(task) is dict:
                 awsy_test = task["task"]["extra"].get("suite", [])
                 run_on_projects = task["attributes"]["run_on_projects"]
             else:
@@ -499,10 +678,10 @@ class AwsyGatherer(FrameworkGatherer):
         self._generate_ci_tasks()
         return {
             "Awsy tests": {
-                "tp6": "",
-                "base": "",
-                "dmd": "",
-                "tp5": "",
+                "tp6": {},
+                "base": {},
+                "dmd": {},
+                "tp5": {},
             }
         }
 
@@ -511,10 +690,15 @@ class AwsyGatherer(FrameworkGatherer):
             title.capitalize(), content, header_type="H4"
         )
 
-    def build_test_description(self, title, test_description="", suite_name=""):
+    def build_test_description(
+        self, title, test_description="", suite_name="", metrics_info=None
+    ):
         dropdown_suite_name = suite_name.replace(" ", "-")
         result = f".. dropdown:: {title} ({test_description})\n"
-        result += f"   :container: + anchor-id-{title}-{dropdown_suite_name}\n\n"
+        result += f"   :class-container: anchor-id-{title}-{dropdown_suite_name}\n\n"
+        result += self.build_command_to_run_locally(
+            "awsy-test", "" if title == "tp6" else f"--{title}"
+        )
 
         awsy_data = read_yaml(self._yaml_path)["suites"]["Awsy tests"]
         if "owner" in awsy_data.keys():
